@@ -3,7 +3,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from grizzly.common import TestCase
+from grizzly.common.storage import TestCase
 
 from . import NoOpAdapter
 
@@ -14,6 +14,6 @@ def test_no_op_01():
     adapter.setup(None, None)
     test = TestCase("a", "b", adapter.name)
     assert not test.data_size
-    assert not test.contains("a")
+    assert "a" not in test.contents
     adapter.generate(test, None)
-    assert test.contains("a")
+    assert "a" in test.contents
