@@ -4,10 +4,12 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from argparse import ArgumentParser, HelpFormatter
 from logging import CRITICAL, DEBUG, ERROR, INFO, WARNING
-from os.path import exists, isdir, isfile
+from os.path import exists, isfile
+from pathlib import Path
 from platform import system
 
 from .common.plugins import scan as scan_plugins
+from .common.plugins import scan_target_assets
 from .common.utils import TIMEOUT_DELAY
 
 
@@ -31,7 +33,7 @@ class SortingHelpFormatter(HelpFormatter):
 
 class CommonArgs:
     IGNORABLE = ("log-limit", "memory", "timeout")
-    IGNORE = ("log-limit", "timeout")
+    DEFAULT_IGNORE = ("log-limit", "timeout")
 
     def __init__(self):
         # log levels for console logging
@@ -49,24 +51,59 @@ class CommonArgs:
                 formatter_class=SortingHelpFormatter, conflict_handler="resolve"
             )
 
+        targets = scan_plugins("grizzly_targets")
+        if not targets:
+            self.parser.error("No Platforms (Targets) are installed")
+
         self.parser.add_argument("binary", help="Firefox binary to run")
         self.parser.add_argument(
             "--log-level",
+            choices=sorted(self._level_map),
             default="INFO",
-            help="Configure console logging. Options: %s (default: %%(default)s)"
-            % ", ".join(
-                k for k, v in sorted(self._level_map.items(), key=lambda x: x[1])
-            ),
+            help="Configure console logging (default: %(default)s)",
         )
 
+        # build 'asset' help string
+        assets = scan_target_assets()
+        asset_msg = list()
+        for target in sorted(assets):
+            if assets[target]:
+                asset_msg.append(
+                    "%s: %s. " % (target, ", ".join(sorted(assets[target])))
+                )
+
         self.launcher_grp = self.parser.add_argument_group("Launcher Arguments")
+        self.launcher_grp.add_argument(
+            "--asset",
+            action="append",
+            default=list(),
+            metavar=("ASSET", "PATH"),
+            nargs=2,
+            help="Specify target specific asset files. %s" % ("".join(asset_msg),),
+        )
         self.launcher_grp.add_argument(
             "-e",
             "--extension",
-            action="append",
-            help="Install an extension. Specify the path to the xpi or the directory"
-            " containing the unpacked extension. To install multiple extensions"
-            " specify multiple times",
+            help="DEPRECATED. Install an extension. Specify the path to the xpi or the"
+            " directory containing the unpacked extension.",
+        )
+        headless_choices = ["default"]
+        if system().startswith("Linux"):
+            headless_choices.append("xvfb")
+        self.launcher_grp.add_argument(
+            "--headless",
+            choices=headless_choices,
+            const="default",
+            default=None,
+            nargs="?",
+            help="Headless mode. 'default' uses browser's built-in headless mode.",
+        )
+        self.launcher_grp.add_argument(
+            "--launch-attempts",
+            type=int,
+            default=3,
+            help="Number of attempts to launch the browser before LaunchError is raised"
+            " (default: %(default)s)",
         )
         self.launcher_grp.add_argument(
             "--launch-timeout",
@@ -91,10 +128,12 @@ class CommonArgs:
         self.launcher_grp.add_argument(
             "--platform",
             default="ffpuppet",
-            help="Installed Platforms (Targets): %s (default: %%(default)s)"
-            % ", ".join(sorted(scan_plugins("grizzly_targets"))),
+            choices=sorted(targets),
+            help="Target to use (default: %(default)s)",
+        )
+        self.launcher_grp.add_argument(
+            "-p", "--prefs", help="DEPRECATED. prefs.js file to use"
         )
-        self.launcher_grp.add_argument("-p", "--prefs", help="prefs.js file to use")
         self.launcher_grp.add_argument(
             "--relaunch",
             type=int,
@@ -124,7 +163,7 @@ class CommonArgs:
         )
         if system().startswith("Linux"):
             self.launcher_grp.add_argument(
-                "--xvfb", action="store_true", help="Use Xvfb."
+                "--xvfb", action="store_true", help="DEPRECATED. Use Xvfb."
             )
         else:
             self.parser.set_defaults(xvfb=False)
@@ -136,9 +175,19 @@ class CommonArgs:
         self.reporter_grp.add_argument(
             "--ignore",
             nargs="*",
-            default=list(self.IGNORE),
-            help="Space separated list of issue types to ignore. Valid options: %s"
-            " (default: %s)" % (" ".join(self.IGNORABLE), " ".join(self.IGNORE)),
+            choices=self.IGNORABLE,
+            default=self.DEFAULT_IGNORE,
+            metavar="IGNORABLE",
+            help="Space-separated list of ignorable types. Pass zero args to disable."
+            " Available: %s (default: %s)"
+            % (" ".join(self.IGNORABLE), " ".join(self.DEFAULT_IGNORE)),
+        )
+        self.reporter_grp.add_argument(
+            "-l",
+            "--logs",
+            default=Path.cwd(),
+            type=Path,
+            help="Location to save logs and test cases. (default: %(default)s)",
         )
         self.reporter_grp.add_argument(
             "--tool",
@@ -174,29 +223,22 @@ class CommonArgs:
         return args
 
     def sanity_check(self, args):
-        targets = scan_plugins("grizzly_targets")
-        if not targets:
-            self.parser.error("No Platforms (Targets) are installed")
-
         if "binary" not in self._sanity_skip and not isfile(args.binary):
             self.parser.error("file not found: %r" % (args.binary,))
 
-        # sanitize ignore list
-        args.ignore = {arg.lower() for arg in args.ignore}
-        for ignore in args.ignore:
-            if ignore not in self.IGNORABLE:
-                self.parser.error("Unrecognized ignore value %r" % (ignore,))
+        if args.launch_attempts < 1:
+            self.parser.error("--launch-attempts must be >= 1")
 
-        # check log level
-        log_level = self._level_map.get(args.log_level.upper(), None)
-        if log_level is None:
-            self.parser.error("Invalid log-level %r" % (args.log_level,))
-        args.log_level = log_level
+        args.log_level = self._level_map[args.log_level]
 
         if args.log_limit < 0:
             self.parser.error("--log-limit must be >= 0")
         args.log_limit *= 1_048_576
 
+        # if logs is specified, we need it to be a directory (whether existent or not)
+        if args.logs and args.logs.is_file():
+            self.parser.error("--logs cannot be a file")
+
         if args.memory < 0:
             self.parser.error("--memory must be >= 0")
         args.memory *= 1_048_576
@@ -204,18 +246,33 @@ class CommonArgs:
         if args.relaunch < 1:
             self.parser.error("--relaunch must be >= 1")
 
-        if args.extension:
-            for ext in args.extension:
-                if not exists(ext):
-                    self.parser.error("%r does not exist" % (ext,))
-                if not isdir(ext) or (isfile(ext) and ext.endswith(".xpi")):
-                    self.parser.error("Extension must be a folder or .xpi")
+        if args.pernosco or args.rr:
+            # currently we only support rr on Linux
+            settings = "/proc/sys/kernel/perf_event_paranoid"
+            value = int(Path(settings).read_text())
+            if value > 1:
+                self.parser.error("rr needs %s <= 1, but it is %d" % (settings, value))
 
-        if args.platform not in targets:
-            self.parser.error("Platform %r not installed" % (args.platform,))
+        # TODO: remove deprecated 'extension' from args
+        if args.extension:  # pragma: no cover
+            args.asset.append(["extension", args.extension])
 
-        if args.prefs and not isfile(args.prefs):
-            self.parser.error("--prefs file not found")
+        # TODO: remove deprecated 'prefs' from args
+        if args.prefs:  # pragma: no cover
+            args.asset.append(["prefs", args.prefs])
+
+        # check args.platform before args.asset since it is used
+        if args.asset:
+            supported_assets = scan_target_assets()[args.platform]
+            for asset, path in args.asset:
+                if not supported_assets or asset not in supported_assets:
+                    self.parser.error(
+                        "Asset %r not supported by target %r" % (asset, args.platform)
+                    )
+                if not exists(path):
+                    self.parser.error(
+                        "Failed to add asset %r cannot find %r" % (asset, path)
+                    )
 
         if args.time_limit is not None and args.time_limit < 1:
             self.parser.error("--time-limit must be >= 1")
@@ -227,15 +284,21 @@ class CommonArgs:
             if args.tool is not None and not args.fuzzmanager:
                 self.parser.error("--tool can only be given with --fuzzmanager")
 
+        if args.xvfb:  # pragma: no cover
+            args.headless = "xvfb"
+
 
 class GrizzlyArgs(CommonArgs):
     def __init__(self):
         super().__init__()
+
+        adapters = scan_plugins("grizzly_adapters")
+        if not adapters:
+            self.parser.error("No Adapters are installed")
+
         self._sanity_skip.add("tool")
         self.parser.add_argument(
-            "adapter",
-            help="Installed Adapters: %s"
-            % ", ".join(sorted(scan_plugins("grizzly_adapters"))),
+            "adapter", choices=sorted(adapters), help="Adapter to use."
         )
         self.parser.add_argument(
             "--enable-profiling",
@@ -244,13 +307,22 @@ class GrizzlyArgs(CommonArgs):
             " status reporter while running Grizzly.",
         )
         self.parser.add_argument(
-            "-i", "--input", help="Test case or directory containing test cases."
+            "-i",
+            "--input",
+            type=Path,
+            help="Test case or directory containing test cases.",
         )
         self.parser.add_argument(
             "--limit",
             type=int,
             default=0,
-            help="Maximum number of iterations to be performed. (default: no limit)",
+            help="Maximum number of iterations to be performed. (default: 'no limit')",
+        )
+        self.parser.add_argument(
+            "--smoke-test",
+            action="store_true",
+            help="Perform a small number of iterations to check if everything is"
+            " working as expected. Exit immediately if a result is found.",
         )
         self.parser.add_argument(
             "-v",
@@ -281,6 +353,16 @@ class GrizzlyArgs(CommonArgs):
             help="Maximum number of test cases to include in the report."
             " (default: %(default)s)",
         )
+        self.reporter_grp.add_argument(
+            "--limit-reports",
+            type=int,
+            default=5,
+            help="Maximum number of times a unique result will be submitted."
+            " This includes results submitted by parallel and previously run"
+            " (within 24h) processes. This can help avoid spamming duplicate results."
+            " The first time a result is seen it will always be submitted."
+            " (default: %(default)s) - Use 0 for 'no limit'",
+        )
         self.reporter_grp.add_argument(
             "--s3-fuzzmanager",
             action="store_true",
@@ -290,12 +372,6 @@ class GrizzlyArgs(CommonArgs):
 
     def sanity_check(self, args):
         super().sanity_check(args)
-        adapters = scan_plugins("grizzly_adapters")
-        if not adapters:
-            self.parser.error("No Adapters are installed")
-
-        if args.adapter not in adapters:
-            self.parser.error("Adapter %r is not installed" % (args.adapter,))
 
         if args.collect < 1:
             self.parser.error("--collect must be greater than 0")
@@ -305,12 +381,15 @@ class GrizzlyArgs(CommonArgs):
                 "--fuzzmanager and --s3-fuzzmanager are mutually exclusive"
             )
 
-        if args.input and not exists(args.input):
-            self.parser.error("%r does not exist" % (args.input,))
+        if args.input and not args.input.exists():
+            self.parser.error("'%s' does not exist" % (args.input,))
 
         if args.limit < 0:
             self.parser.error("--limit must be >= 0")
 
+        if args.limit_reports < 0:
+            self.parser.error("--limit-reports must be >= 0")
+
         if args.runtime < 0:
             self.parser.error("--runtime must be >= 0")
 
