@@ -11,44 +11,207 @@ from unittest.mock import Mock
 
 from pytest import mark, raises
 
-from .status_reporter import Status, StatusReporter, TracebackReport, main
+from .status_reporter import (
+    ReductionStatus,
+    ReductionStatusReporter,
+    Status,
+    StatusReporter,
+    TracebackReport,
+    main,
+)
 
 GBYTES = 1_073_741_824
 
 
 def _fake_sys_info():
-    return (
-        "CPU & Load : 64 @ 93.1% (85.25, 76.21, 51.06)\n"
-        "    Memory : 183.9GB of 251.9GB free\n"
-        "      Disk : 22.2GB of 28.7GB free"
+    return [
+        ("CPU & Load", "64 @ 93% (85.25, 76.21, 51.06)"),
+        ("Memory", "183.9GB of 251.9GB free"),
+        ("Disk", "22.2GB of 28.7GB free"),
+    ]
+
+
+def test_reduce_status_reporter_01():
+    """test basic ReductionStatusReporter"""
+    st_rpt = ReductionStatusReporter(None)
+    assert not st_rpt.has_results
+    st_rpt._sys_info = _fake_sys_info
+    assert "No status reports available" in st_rpt.specific()
+    assert "No status reports available" in st_rpt.summary()
+
+
+def test_reduce_status_reporter_02(mocker, tmp_path):
+    """test ReductionStatusReporter.load()"""
+    # missing reports path
+    st_rpt = ReductionStatusReporter.load(str(tmp_path / "status.db"))
+    assert not st_rpt.reports
+
+    # empty reports and tb paths
+    st_rpt = ReductionStatusReporter.load(
+        str(tmp_path / "status.db"), tb_path=str(tmp_path)
     )
+    assert not st_rpt.reports
+    assert isinstance(st_rpt.tracebacks, list)
+    assert not st_rpt.tracebacks
+
+    # multiple reports
+    size_cb = mocker.Mock(side_effect=count(start=1000, step=-100))
+    db_file = str(tmp_path / "status.db")
+    ReductionStatus.start(
+        db_file=db_file,
+        testcase_size_cb=size_cb,
+    )
+    status2 = ReductionStatus(
+        db_file=db_file,
+        pid=1,
+        testcase_size_cb=size_cb,
+    )
+    status2.report(force=True)
+    st_rpt = ReductionStatusReporter.load(db_file)
+    assert len(st_rpt.reports) > 1
 
 
-def test_status_reporter_01(tmp_path):
+def test_reduce_status_reporter_03(mocker, tmp_path):
+    """test ReductionStatusReporter.summary()"""
+    mocker.patch("grizzly.common.status.getpid", side_effect=(1, 2))
+    mocker.patch("grizzly.common.status.time", side_effect=count(start=1.0, step=1.0))
+    size_cb = mocker.Mock(side_effect=count(start=1000, step=-100))
+    db_file = str(tmp_path / "status.db")
+    # single report
+    status = ReductionStatus.start(
+        db_file=db_file,
+        testcase_size_cb=size_cb,
+        crash_id=123,
+        tool="fuzzmatic",
+    )
+    status.analysis["ran"] = True
+    status.run_params["speed"] = 123.0
+    status.signature_info["info"] = "crash"
+    status.last_reports.append(45678)
+    status.record("init")
+    with status.measure("total"):
+        with status.measure("strategy_0"):
+            status.attempts += 1
+            status.successes += 1
+            status.iterations += 1
+        with status.measure("strategy_1"):
+            status.attempts += 3
+            status.successes += 1
+            status.iterations += 3
+    status.report(force=True)
+
+    rptr = ReductionStatusReporter.load(db_file)
+    rptr._sys_info = _fake_sys_info
+    assert rptr.reports
+    output = rptr.summary(sysinfo=True, timestamp=True)
+    assert "duration" in output
+    assert "successes" in output
+    assert "attempts" in output
+    assert "init" in output
+    assert "strategy_0" in output
+    assert "strategy_1" in output
+    assert "total" in output
+    assert "Timestamp" in output
+    assert len(output.splitlines()) == 16
+
+
+def test_reduce_status_reporter_04(mocker, tmp_path):
+    """test ReductionStatusReporter.specific()"""
+    mocker.patch("grizzly.common.status.getpid", side_effect=(1, 2))
+    db_file = str(tmp_path / "status.db")
+    # single report
+    status = ReductionStatus.start(
+        db_file=db_file,
+        strategies=["strategy_0"],
+        testcase_size_cb=lambda: 47,
+        crash_id=12,
+        tool="fuzzmatic",
+    )
+    assert status.original is None
+    rptr = ReductionStatusReporter.load(db_file)
+    assert rptr.reports
+    output = rptr.specific()
+    assert len(output.splitlines()) == 1
+    status.analysis["ran"] = True
+    status.run_params["splines"] = "reticulated"
+    status.last_reports.append(45678)
+    status.record("init")
+    with status.measure("total"):
+        with status.measure("strategy_0"):
+            status.iterations = 1
+            status.attempts = 1
+            status.successes = 1
+    status.report(force=True)
+    rptr = ReductionStatusReporter.load(db_file)
+    assert rptr.reports
+    output = rptr.specific()
+    assert len(output.splitlines()) == 8
+    assert "Analysis" in output
+    assert "Run Parameters" in output
+    assert "Current Strategy" in output
+    assert "Current/Original" in output
+    assert "Results" in output
+    assert "Time Elapsed" in output
+
+
+def test_reduce_status_reporter_05(tmp_path):
+    """test ReductionStatusReporter.load() with traceback"""
+    db_file = str(tmp_path / "status.db")
+    status = ReductionStatus.start(
+        db_file=db_file,
+        testcase_size_cb=lambda: 47,
+    )
+    with status.measure("total"):
+        status.iteration = 1
+    status.report(force=True)
+    # create boring screenlog
+    (tmp_path / "screenlog.0").write_bytes(b"boring\ntest\n123\n")
+    # create first screenlog
+    with (tmp_path / "screenlog.1").open("wb") as test_fp:
+        test_fp.write(b"Traceback (most recent call last):\n")
+        test_fp.write(b"  blah\n")
+        test_fp.write(b"IndexError: list index out of range\n")
+    rptr = StatusReporter.load(db_file, tb_path=str(tmp_path))
+    assert len(rptr.tracebacks) == 1
+    # create second screenlog
+    with (tmp_path / "screenlog.1234").open("wb") as test_fp:
+        test_fp.write(b"Traceback (most recent call last):\n")
+        test_fp.write(b"  blah\n")
+        test_fp.write(b"foo.bar.error: blah\n")
+    rptr = StatusReporter.load(db_file, tb_path=str(tmp_path))
+    assert len(rptr.tracebacks) == 2
+    # create third screenlog
+    with (tmp_path / "screenlog.3").open("wb") as test_fp:
+        test_fp.write(b"Traceback (most recent call last):\n")
+        test_fp.write(b"  blah\n")
+        test_fp.write(b"KeyboardInterrupt\n")
+    rptr = ReductionStatusReporter.load(db_file, tb_path=str(tmp_path))
+    assert len(rptr.tracebacks) == 2
+    merged_log = rptr.summary()
+    assert len(merged_log.splitlines()) == 13
+    assert "screenlog.1" in merged_log
+    assert "screenlog.1234" in merged_log
+    assert "IndexError" in merged_log
+    assert "foo.bar.error" in merged_log
+    assert "screenlog.3" not in merged_log
+
+
+def test_status_reporter_01():
     """test basic StatusReporter"""
     st_rpt = StatusReporter(list())
     assert not st_rpt.has_results
     st_rpt._sys_info = _fake_sys_info
-    assert "No status reports available" in st_rpt._specific()
-    report = tmp_path / "output.txt"
-    st_rpt.dump_specific(str(report))
-    assert report.is_file()
-    st_rpt.print_specific()
-    assert "No status reports available" in st_rpt._summary()
-    report.unlink()
-    st_rpt.dump_summary(str(report))
-    assert report.is_file()
-    st_rpt.print_summary()
+    assert "No status reports available" in st_rpt.specific()
+    assert "No status reports available" in st_rpt.summary()
 
 
 def test_status_reporter_02(tmp_path):
     """test StatusReporter.load()"""
-    Status.PATH = str(tmp_path / "missing")
     # missing reports path
-    st_rpt = StatusReporter.load()
+    st_rpt = StatusReporter.load(str(tmp_path / "status.db"))
     assert not st_rpt.reports
     # empty reports and tb paths
-    st_rpt = StatusReporter.load(tb_path=str(tmp_path))
+    st_rpt = StatusReporter.load(str(tmp_path / "status.db"), tb_path=str(tmp_path))
     assert isinstance(st_rpt.reports, list)
     assert not st_rpt.reports
     assert isinstance(st_rpt.tracebacks, list)
@@ -67,7 +230,7 @@ def test_status_reporter_02(tmp_path):
         (
             Mock(free=12, total=GBYTES),
             Mock(available=12, total=GBYTES),
-            lambda: "(0.12, 0.34, 0.56)",
+            lambda: (0.12, 0.34, 0.56),
         ),
     ],
 )
@@ -95,21 +258,18 @@ def test_status_reporter_03(mocker, disk, memory, getloadavg):
             "grizzly.common.status_reporter.getloadavg", side_effect=getloadavg
         )
     sysinfo = StatusReporter._sys_info()
-    if disk.free < GBYTES or memory.available < GBYTES:
-        assert "MB" in sysinfo
-    else:
-        assert "MB" not in sysinfo
-    lines = sysinfo.split("\n")
-    assert len(lines) == 3
-    assert "CPU & Load : " in lines[0]
+    assert len(sysinfo) == 3
+    assert sysinfo[0][0] == "CPU & Load"
+    assert sysinfo[1][0] == "Memory"
+    assert sysinfo[2][0] == "Disk"
     if getloadavg is not None:
-        assert lines[0].endswith("(0.12, 0.34, 0.56)")
-    assert "Memory : " in lines[1]
-    assert "Disk : " in lines[2]
-    # verify alignment
-    position = len(lines[0].split(":")[0])
-    for line in lines:
-        assert line[position] == ":"
+        assert sysinfo[0][-1].endswith(" (0.1, 0.3, 0.6)")
+    if disk.free < GBYTES or memory.available < GBYTES:
+        assert "MB" in sysinfo[1][-1]
+        assert "MB" in sysinfo[2][-1]
+    else:
+        assert "MB" not in sysinfo[1][-1]
+        assert "MB" not in sysinfo[2][-1]
 
 
 def test_status_reporter_04(tmp_path):
@@ -126,48 +286,51 @@ def test_status_reporter_04(tmp_path):
 
 
 def test_status_reporter_05(mocker, tmp_path):
-    """test StatusReporter._summary()"""
+    """test StatusReporter.summary()"""
+    mocker.patch("grizzly.common.status.getpid", side_effect=(1, 2))
     mocker.patch("grizzly.common.status.time", side_effect=count(start=1.0, step=1.0))
-    mocker.patch("grizzly.common.status_reporter.time", side_effect=(1.0, 2.0))
-    Status.PATH = str(tmp_path)
+    db_file = str(tmp_path / "status.db")
     # single report
-    status = Status.start()
+    status = Status.start(db_file=db_file)
     status.ignored = 0
     status.iteration = 1
     status.log_size = 0
     status.report(force=True)
-    rptr = StatusReporter.load()
+    rptr = StatusReporter.load(db_file)
     rptr._sys_info = _fake_sys_info
     assert rptr.reports is not None
     assert len(rptr.reports) == 1
-    output = rptr._summary(runtime=False)
+    output = rptr.summary(runtime=False)
     assert "Iteration" in output
     assert "Rate" in output
     assert "Results" in output
-    assert "ignored" not in output
+    assert "Blockers" not in output
+    assert "Ignored" not in output
     assert "Logs" not in output
     assert "Runtime" not in output
     assert "Timestamp" not in output
     assert len(output.split("\n")) == 3
     # multiple reports
-    status = Status.start()
+    status = Status.start(db_file=db_file)
     status.ignored = 1
     status.iteration = 8
     status.log_size = 86900000
+    status.results.count("test", "test")
+    status.results.count("test", "test")
     status.report(force=True)
-    rptr = StatusReporter.load()
+    rptr = StatusReporter.load(db_file)
     rptr._sys_info = _fake_sys_info
     assert len(rptr.reports) == 2
-    output = rptr._summary(sysinfo=True, timestamp=True)
+    output = rptr.summary(sysinfo=True, timestamp=True)
     assert "Iteration" in output
     assert "Rate" in output
     assert "Results" in output
-    assert "ignored" in output
+    assert "Ignored" in output
     assert "Logs" in output
     assert "Runtime" in output
     assert "Timestamp" in output
     lines = output.split("\n")
-    assert len(lines) == 9
+    assert len(lines) == 10
     # verify alignment
     position = len(lines[0].split(":")[0])
     for line in lines:
@@ -175,104 +338,92 @@ def test_status_reporter_05(mocker, tmp_path):
 
 
 def test_status_reporter_06(mocker, tmp_path):
-    """test StatusReporter._specific()"""
-    Status.PATH = str(tmp_path)
-    mocker.patch("grizzly.common.status_reporter.time", return_value=3661.0)
+    """test StatusReporter.specific()"""
+    mocker.patch("grizzly.common.status.getpid", side_effect=(1, 2))
+    db_file = str(tmp_path / "status.db")
     # single report
-    mocker.patch(
-        "grizzly.common.status.time", side_effect=count(start=0.0, step=3600.0)
-    )
-    status = Status.start()
+    status = Status.start(db_file=db_file)
     status.ignored = 0
     status.iteration = 1
     status.log_size = 0
     status.report(force=True)
-    rptr = StatusReporter.load()
+    rptr = StatusReporter.load(db_file)
     assert rptr.reports is not None
-    output = rptr._specific()
-    assert len(output.split("\n")[:-1]) == 2
-    assert "Ignored" in output
-    assert "Iteration" in output
+    output = rptr.specific()
+    assert len(output.strip().split("\n")) == 4
+    assert "Ignored" not in output
+    assert "Iterations" in output
     assert "Results" in output
-    assert "EXPIRED" not in output
+    assert "(Blockers detected)" not in output
+    assert "Runtime" in output
     # multiple reports
-    mocker.patch(
-        "grizzly.common.status.time", side_effect=count(start=0.0, step=3661.0)
-    )
-    status = Status.start(enable_profiling=True)
+    status = Status.start(db_file=db_file, enable_profiling=True)
     status.ignored = 1
-    status.iteration = 432422
-    status._results = {"sig": 123}
+    status.iteration = 50
+    status.results.count("uid1", "sig1")
+    status.results.count("uid1", "sig1")
+    status.results.count("uid1", "sig1")
     status.record("test1", 0.91)
     status.record("test1", 1.0)
     status.record("test1", 1.23456)
     status.record("test2", 1201.1)
     status.report(force=True)
-    rptr = StatusReporter.load()
+    rptr = StatusReporter.load(db_file)
     assert len(rptr.reports) == 2
-    output = rptr._specific()
-    assert len(output.split("\n")[:-1]) == 7
+    output = rptr.specific()
+    assert len(output.strip().split("\n")) == 13
     assert "Ignored" in output
-    assert "Iteration" in output
+    assert "Iterations" in output
     assert "Results" in output
+    assert "Runtime" in output
+    assert "(Blockers detected)" in output
     assert "Profiling entries" in output
-    assert "> test1:" in output
-    assert "> test2:" in output
-    assert "EXPIRED" not in output
-    # expired report
-    mocker.patch("grizzly.common.status.time", return_value=1.0)
-    status = Status.start()
-    status.ignored = 1
-    status.iteration = 1
-    status.report(force=True)
-    rptr = StatusReporter.load()
-    assert len(rptr.reports) == 3
-    output = rptr._specific()
-    assert len(output.split("\n")[:-1]) == 8
-    assert "EXPIRED" in output
+    assert "test1" in output
+    assert "test2" in output
 
 
-def test_status_reporter_07(tmp_path):
-    """test StatusReporter._results()"""
-    Status.PATH = str(tmp_path)
+def test_status_reporter_07(mocker, tmp_path):
+    """test StatusReporter.results()"""
+    mocker.patch("grizzly.common.status.getpid", side_effect=(1, 2, 3))
+    db_file = str(tmp_path / "status.db")
     # single report without results
-    status = Status.start()
+    status = Status.start(db_file=db_file)
     status.ignored = 0
     status.iteration = 1
     status.log_size = 0
     status.report(force=True)
-    rptr = StatusReporter.load()
+    rptr = StatusReporter.load(db_file)
     assert rptr.reports is not None
     assert len(rptr.reports) == 1
     assert not rptr.has_results
-    assert not rptr._results()
+    assert rptr.results() == "No results available\n"
     # multiple reports with results
-    status = Status.start()
+    status = Status.start(db_file=db_file)
     status.iteration = 1
-    status.count_result("[@ test1]")
-    status.count_result("[@ test2]")
-    status.count_result("[@ test1]")
+    status.results.count("uid1", "[@ test1]")
+    status.results.count("uid2", "[@ test2]")
+    status.results.count("uid1", "[@ test1]")
     status.report(force=True)
-    status = Status.start()
+    status = Status.start(db_file=db_file)
     status.iteration = 1
-    status.count_result("[@ test1]")
-    status.count_result("[@ longsignature123]")
+    status.results.count("uid1", "[@ test1]")
+    status.results.count("uid3", "[@ longsignature123]")
     status.report(force=True)
-    rptr = StatusReporter.load()
+    rptr = StatusReporter.load(db_file)
     assert rptr.has_results
     assert len(rptr.reports) == 3
-    output = rptr._results(max_len=16)
-    assert "3: '[@ test1]'" in output
-    assert "1: '[@ test2]'" in output
-    assert "1: '[@ longsignature...'" in output
-    assert len(output.split("\n")[:-1]) == 3
+    output = rptr.results(max_len=19)
+    assert "3 : [@ test1]" in output
+    assert "1 : [@ test2]" in output
+    assert "1 : [@ longsignature..." in output
+    assert "(* = Blocker)" in output
+    assert len(output.strip().split("\n")) == 4
 
 
 def test_status_reporter_08(tmp_path):
     """test StatusReporter.load() with traceback"""
-    (tmp_path / "status").mkdir()
-    Status.PATH = str(tmp_path / "status")
-    status = Status.start()
+    db_file = str(tmp_path / "status.db")
+    status = Status.start(db_file=db_file)
     status.ignored = 0
     status.iteration = 1
     status.log_size = 0
@@ -284,23 +435,23 @@ def test_status_reporter_08(tmp_path):
         test_fp.write(b"Traceback (most recent call last):\n")
         test_fp.write(b"  blah\n")
         test_fp.write(b"IndexError: list index out of range\n")
-    rptr = StatusReporter.load(tb_path=str(tmp_path))
+    rptr = StatusReporter.load(db_file, tb_path=str(tmp_path))
     assert len(rptr.tracebacks) == 1
     # create second screenlog
     with (tmp_path / "screenlog.1234").open("wb") as test_fp:
         test_fp.write(b"Traceback (most recent call last):\n")
         test_fp.write(b"  blah\n")
         test_fp.write(b"foo.bar.error: blah\n")
-    rptr = StatusReporter.load(tb_path=str(tmp_path))
+    rptr = StatusReporter.load(db_file, tb_path=str(tmp_path))
     assert len(rptr.tracebacks) == 2
     # create third screenlog
     with (tmp_path / "screenlog.3").open("wb") as test_fp:
         test_fp.write(b"Traceback (most recent call last):\n")
         test_fp.write(b"  blah\n")
         test_fp.write(b"KeyboardInterrupt\n")
-    rptr = StatusReporter.load(tb_path=str(tmp_path))
+    rptr = StatusReporter.load(db_file, tb_path=str(tmp_path))
     assert len(rptr.tracebacks) == 2
-    merged_log = rptr._summary()
+    merged_log = rptr.summary()
     assert len(merged_log.splitlines()) == 14
     assert "screenlog.1" in merged_log
     assert "screenlog.1234" in merged_log
@@ -311,38 +462,38 @@ def test_status_reporter_08(tmp_path):
 
 def test_status_reporter_09(tmp_path):
     """test StatusReporter.load() no reports with traceback"""
-    (tmp_path / "status").mkdir()
-    Status.PATH = str(tmp_path / "status")
     # create screenlog with tb
     with (tmp_path / "screenlog.1").open("wb") as test_fp:
         test_fp.write(b"Traceback (most recent call last):\n")
         test_fp.write(b"  blah\n")
         test_fp.write(b"IndexError: list index out of range\n")
-    rptr = StatusReporter.load(tb_path=str(tmp_path))
+    rptr = StatusReporter.load(str(tmp_path / "status.db"), tb_path=str(tmp_path))
     rptr._sys_info = _fake_sys_info
     assert len(rptr.tracebacks) == 1
-    output = rptr._summary()
+    output = rptr.summary()
     assert len(output.splitlines()) == 7
     assert "No status reports available" in output
     assert "IndexError" in output
 
 
-def test_status_reporter_10(tmp_path):
+def test_status_reporter_10(mocker, tmp_path):
     """test StatusReporter.summary() limit with traceback"""
-    (tmp_path / "status").mkdir()
-    Status.PATH = str(tmp_path / "status")
+    mocker.patch("grizzly.common.status.getpid", side_effect=(1, 2))
+    db_file = str(tmp_path / "status.db")
     # create reports
-    status = Status.start()
+    status = Status.start(db_file=db_file)
     status.ignored = 100
     status.iteration = 1000
     status.log_size = 9999999999
-    status._results = {"sig": 123}
+    status.results.count("uid1", "[@ sig1]")
+    status.results._count["uid1"] = 123
     status.report(force=True)
-    status = Status.start()
+    status = Status.start(db_file=db_file)
     status.ignored = 9
     status.iteration = 192938
     status.log_size = 0
-    status._results = {"sig": 3}
+    status.results.count("uid2", "[@ sig2]")
+    status.results._count["uid2"] = 3
     status.report(force=True)
     # create screenlogs with tracebacks
     for i in range(10):
@@ -354,13 +505,26 @@ def test_status_reporter_10(tmp_path):
                 )
                 test_fp.write(b"    some_long_name_for_a_func_%04d()\n" % (j,))
             test_fp.write(b"IndexError: list index out of range\n")
-    rptr = StatusReporter.load(tb_path=str(tmp_path))
+    rptr = StatusReporter.load(db_file, tb_path=str(tmp_path))
     rptr._sys_info = _fake_sys_info
     assert len(rptr.tracebacks) == 10
-    merged_log = rptr._summary(runtime=True, sysinfo=True, timestamp=True)
+    merged_log = rptr.summary(
+        runtime=True, sysinfo=True, timestamp=True, iters_per_result=1
+    )
     assert len(merged_log) < StatusReporter.SUMMARY_LIMIT
 
 
+def test_status_reporter_11():
+    """test StatusReporter.format_entries()"""
+    assert StatusReporter.format_entries([]) == ""
+    assert StatusReporter.format_entries([("test", None)]) == "test"
+    assert StatusReporter.format_entries([("test", "1")]) == "test : 1"
+    out = StatusReporter.format_entries(
+        [("first", "1"), ("second", "2"), ("third", "3")]
+    )
+    assert out == " first : 1\nsecond : 2\n third : 3"
+
+
 def test_traceback_report_01():
     """test simple TracebackReport"""
     tbr = TracebackReport("log.txt", ["0", "1", "2"], prev_lines=["-2", "-1"])
@@ -413,7 +577,7 @@ def test_traceback_report_03(tmp_path):
         test_fp.write(b"    a = b[10]\n")
         test_fp.write(b"foo.bar.error: blah\n")
         test_fp.write(b"end junk\n")
-    tbr = TracebackReport.from_file(str(test_log), max_preceeding=0)
+    tbr = TracebackReport.from_file(str(test_log), max_preceding=0)
     assert len(tbr.lines) == 6
     assert not tbr.prev_lines
     assert not tbr.is_kbi
@@ -532,41 +696,125 @@ def test_traceback_report_08(tmp_path):
     assert "AssertionError" in output
 
 
-def test_main_01(tmp_path):
+@mark.usefixtures("tmp_path_status_db")
+def test_main_01(mocker):
     """test main() with no reports"""
-    Status.PATH = str(tmp_path)
-    StatusReporter.CPU_POLL_INTERVAL = 0.01
+    mocker.patch(
+        "grizzly.common.status_reporter.StatusReporter.CPU_POLL_INTERVAL", 0.01
+    )
     assert main([]) == 0
 
 
-def test_main_02(tmp_path):
+@mark.usefixtures("tmp_path_status_db")
+def test_main_02(mocker):
     """test main() with a report"""
-    Status.PATH = str(tmp_path)
-    StatusReporter.CPU_POLL_INTERVAL = 0.01
+    mocker.patch(
+        "grizzly.common.status_reporter.StatusReporter.CPU_POLL_INTERVAL", 0.01
+    )
     status = Status.start()
     status.iteration = 1
-    status.count_result("[@ test]")
+    status.results.count("uid", "[@ test]")
     status.report(force=True)
     assert main([]) == 0
 
 
-def test_main_03(tmp_path):
+@mark.parametrize(
+    "report_mode",
+    [
+        "fuzzing",
+        "reducing",
+    ],
+)
+@mark.parametrize(
+    "report_type",
+    [
+        "active",
+        "complete",
+    ],
+)
+@mark.usefixtures("tmp_path_status_db", "tmp_path_reduce_status_db")
+def test_main_03(mocker, tmp_path, report_type, report_mode):
     """test main() --dump"""
-    Status.PATH = str(tmp_path)
-    StatusReporter.CPU_POLL_INTERVAL = 0.01
-    status = Status.start()
-    status.iteration = 1
-    status.report(force=True)
+    mocker.patch(
+        "grizzly.common.status_reporter.StatusReporter.CPU_POLL_INTERVAL", 0.01
+    )
+    if report_mode == "reducing":
+        status = ReductionStatus.start(
+            testcase_size_cb=lambda: 47,
+            strategies=[],
+        )
+        with status.measure("total"):
+            status.iteration = 1
+        status.report(force=True)
+    else:
+        status = Status.start()
+        status.iteration = 1
+        status.report(force=True)
     dump_file = tmp_path / "output.txt"
-    assert main(["--dump", str(dump_file)]) == 0
+    assert (
+        main(
+            [
+                "--dump",
+                str(dump_file),
+                "--type",
+                report_type,
+                "--scan-mode",
+                report_mode,
+            ]
+        )
+        == 0
+    )
     assert dump_file.is_file()
-    assert b"Runtime" not in dump_file.read_bytes()
-    # assert False, dump_file.read_bytes()
+    if report_type == "active":
+        assert b"Runtime" not in dump_file.read_bytes()
+    else:
+        assert b"Timestamp" not in dump_file.read_bytes()
 
 
-def test_main_04():
+@mark.usefixtures("tmp_path_status_db")
+def test_main_04(capsys):
     """test main() with invalid args"""
-    with raises(SystemExit):
-        main(["--mode", "invalid"])
     with raises(SystemExit):
         main(["--tracebacks", "missing"])
+    assert "--tracebacks must be a directory" in capsys.readouterr()[-1]
+
+
+@mark.parametrize(
+    "report_type",
+    [
+        "active",
+        "complete",
+    ],
+)
+@mark.usefixtures("tmp_path_reduce_status_db")
+def test_main_05(mocker, tmp_path, report_type):
+    """test main() --dump"""
+    mocker.patch(
+        "grizzly.common.status_reporter.StatusReporter.CPU_POLL_INTERVAL", 0.01
+    )
+    status = ReductionStatus.start(
+        testcase_size_cb=lambda: 47,
+        strategies=[],
+    )
+    with status.measure("total"):
+        status.iteration = 1
+        status.report(force=True)
+    dump_file = tmp_path / "output.txt"
+    assert (
+        main(
+            [
+                "--dump",
+                str(dump_file),
+                "--type",
+                report_type,
+                "--scan-mode",
+                "reducing",
+            ]
+        )
+        == 0
+    )
+    assert dump_file.is_file()
+    if report_type == "active":
+        assert b"Runtime" not in dump_file.read_bytes()
+    else:
+        assert b"Timestamp" not in dump_file.read_bytes()
