@@ -3,8 +3,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from logging import DEBUG, getLogger
-from os import getcwd, getpid
-from os.path import join as pathjoin
+from os import getpid
 
 from sapphire import Sapphire
 
@@ -15,7 +14,7 @@ from .common.reporter import (
     FuzzManagerReporter,
     S3FuzzManagerReporter,
 )
-from .common.utils import TIMEOUT_DELAY, configure_logging
+from .common.utils import TIMEOUT_DELAY, Exit, configure_logging
 from .session import Session
 from .target import Target, TargetLaunchError, TargetLaunchTimeout
 
@@ -34,10 +33,10 @@ def main(args):
     elif args.s3_fuzzmanager:
         S3FuzzManagerReporter.sanity_check(args.binary)
 
+    if args.headless:
+        LOG.info("Running browser headless (%s)", args.headless)
     if args.ignore:
         LOG.info("Ignoring: %s", ", ".join(args.ignore))
-    if args.xvfb:
-        LOG.info("Running with Xvfb")
     if args.pernosco:
         LOG.info("Running with RR (Pernosco mode)")
     elif args.rr:
@@ -65,7 +64,7 @@ def main(args):
         LOG.info("Using test time limit: %ds, timeout: %ds", time_limit, timeout)
         if timeout < time_limit:
             LOG.error("Timeout must be at least test time limit if not greater")
-            return Session.EXIT_ARGS
+            return Exit.ARGS
         if adapter.HARNESS_FILE and time_limit == timeout:
             LOG.warning(
                 "To avoid relaunches due to tests failing to close"
@@ -81,18 +80,17 @@ def main(args):
         LOG.debug("initializing the Target %r", args.platform)
         target = load_plugin(args.platform, "grizzly_targets", Target)(
             args.binary,
-            args.extension,
             args.launch_timeout,
             args.log_limit,
             args.memory,
+            headless=args.headless,
             pernosco=args.pernosco,
             rr=args.rr,
             valgrind=args.valgrind,
-            xvfb=args.xvfb,
         )
-        if args.prefs:
-            target.prefs = args.prefs
-            LOG.info("Using prefs %r", args.prefs)
+        # add specified assets
+        target.assets.add_batch(args.asset)
+        target.process_assets()
         adapter.monitor = target.monitor
 
         LOG.debug("initializing the Reporter")
@@ -105,11 +103,14 @@ def main(args):
             )
             reporter = S3FuzzManagerReporter(tool=args.tool)
         else:
-            reporter = FilesystemReporter(pathjoin(getcwd(), "results"))
-            LOG.info("Results will be stored in %r", reporter.report_path)
+            reporter = FilesystemReporter(args.logs / "results")
+            LOG.info("Results will be stored in %r", str(reporter.report_path))
+        reporter.display_logs = args.smoke_test or reporter.display_logs
 
-        if args.limit:
-            LOG.info("%r iteration(s) will be attempted", args.limit)
+        # make sure an iteration limit is set if smoke_test is True
+        iteration_limit = (args.limit or 10) if args.smoke_test else args.limit
+        if iteration_limit:
+            LOG.info("%r iteration(s) will be attempted", iteration_limit)
         if args.runtime:
             LOG.info("Runtime is limited to %rs", args.runtime)
 
@@ -128,6 +129,7 @@ def main(args):
                 coverage=args.coverage,
                 enable_profiling=args.enable_profiling,
                 relaunch=relaunch,
+                report_limit=args.limit_reports,
                 report_size=args.collect,
             )
             if args.log_level == DEBUG or args.verbose:
@@ -137,19 +139,21 @@ def main(args):
             session.run(
                 args.ignore,
                 time_limit,
-                input_path=args.input,
-                iteration_limit=args.limit,
+                input_path=str(args.input),
+                iteration_limit=iteration_limit,
+                result_limit=1 if args.smoke_test else 0,
                 runtime_limit=args.runtime,
                 display_mode=display_mode,
+                launch_attempts=args.launch_attempts,
             )
 
     except KeyboardInterrupt:
         LOG.info("Ctrl+C detected.")
-        return Session.EXIT_ABORT
+        return Exit.ABORT
 
     except (TargetLaunchError, TargetLaunchTimeout) as exc:
         LOG.error(str(exc))
-        return Session.EXIT_LAUNCH_FAILURE
+        return Exit.LAUNCH_FAILURE
 
     finally:
         LOG.info("Shutting down...")
@@ -164,4 +168,6 @@ def main(args):
             adapter.cleanup()
         LOG.info("Done.")
 
-    return Session.EXIT_SUCCESS
+    if session and session.status.results.total > 0:
+        return Exit.ERROR
+    return Exit.SUCCESS
