@@ -10,8 +10,6 @@ from locale import LC_ALL, setlocale
 from logging import getLogger
 from math import ceil, log
 from pathlib import Path
-from shutil import rmtree
-from tempfile import mkdtemp
 from time import time
 
 from FTB.Signatures.CrashInfo import CrashSignature
@@ -20,14 +18,14 @@ from sapphire import Sapphire
 
 from ..common.fuzzmanager import CrashEntry
 from ..common.plugins import load as load_plugin
-from ..common.reporter import FilesystemReporter, FuzzManagerReporter
-from ..common.storage import TestCaseLoadFailure, TestFile
-from ..common.utils import ConfigError, configure_logging, grz_tmp
+from ..common.reporter import FilesystemReporter, FuzzManagerReporter, Quality
+from ..common.status import ReductionStatus
+from ..common.status_reporter import ReductionStatusReporter
+from ..common.storage import TestCaseLoadFailure
+from ..common.utils import ConfigError, Exit, configure_logging, grz_tmp
 from ..replay import ReplayManager
-from ..session import Session
 from ..target import Target, TargetLaunchError, TargetLaunchTimeout
 from .exceptions import GrizzlyReduceBaseException, NotReproducible
-from .stats import ReductionStats
 from .strategies import STRATEGIES
 
 __author__ = "Jesse Schwartzentruber"
@@ -77,6 +75,7 @@ class ReduceManager:
         expect_hang=False,
         idle_delay=0,
         idle_threshold=0,
+        reducer_crash_id=None,
         relaunch=1,
         report_period=None,
         report_to_fuzzmanager=False,
@@ -138,7 +137,12 @@ class ReduceManager:
         self._signature = signature
         self._signature_desc = signature_desc
         self._static_timeout = expect_hang or static_timeout
-        self._stats = ReductionStats()
+        self._status = ReductionStatus.start(
+            strategies=strategies,
+            testcase_size_cb=self.testcase_size,
+            crash_id=reducer_crash_id,
+            tool=tool,
+        )
         self._use_analysis = use_analysis
         self._use_harness = use_harness
 
@@ -204,25 +208,27 @@ class ReduceManager:
             LOG.info("Updating max timeout to: %r", new_iter_timeout)
             self.server.timeout = new_iter_timeout
 
-    def run_reliability_analysis(self, stats):
+    def _on_replay_iteration(self):
+        self._status.iterations += 1
+        self._status.report()
+
+    def run_reliability_analysis(self):
         """Run several analysis passes of the current testcase to find `run` parameters.
 
         The number of repetitions and minimum number of crashes are calculated to
         maximize the chances of observing the expected crash.
 
         Arguments:
-            stats (object): Open stats timer (from `ReductionStats.add_timed`).
-                            Pass to `ReductionStats.copy` to properly terminate
-                            the timer for reporting.
+            None
 
         Returns:
-            tuple(int, int, dict): Values for `repeat` and `min_crashes` resulting from
-                                   analysis, and info to include in stats.
+            tuple(int, int): Values for `repeat` and `min_crashes` resulting from
+                             analysis.
         """
+        self._status.report(force=True)
         harness_last_crashes = 0
         harness_crashes = 0
         non_harness_crashes = 0
-        info = {}
 
         # Reset parameters.
         # Use repeat=1 & relaunch=ITERATIONS because this is closer to how we will run
@@ -295,9 +301,9 @@ class ReduceManager:
                     exit_early=False,
                     idle_delay=self._idle_delay,
                     idle_threshold=self._idle_threshold,
+                    on_iteration_cb=self._on_replay_iteration,
                 )
                 try:
-                    stats.add_iterations(replay.status.iteration)
                     crashes = sum(x.count for x in results if x.expected)
                     if crashes and not self._any_crash and self._signature_desc is None:
                         first_expected = next(
@@ -309,7 +315,6 @@ class ReduceManager:
                     self.report(
                         [result for result in results if not result.expected],
                         testcases,
-                        self._stats.copy(stats),
                     )
                     if use_harness:
                         # set harness_crashes in both cases (last_test True/False)
@@ -324,16 +329,24 @@ class ReduceManager:
                     for result in results:
                         result.report.cleanup()
                 reliability = crashes / self.ANALYSIS_ITERATIONS
-                key = ("using" if use_harness else "without") + " harness"
+                desc = ("using" if use_harness else "without") + " harness"
                 if last_test_only:
-                    key += "/last test only"
+                    desc += "/last test only"
+                else:
+                    desc += "/all tests"
                 LOG.info(
                     "Testcase was interesting %0.1f%% of %d attempts %s.",
                     100.0 * reliability,
                     self.ANALYSIS_ITERATIONS,
-                    key,
+                    desc,
                 )
-                info[key] = reliability
+                if use_harness and last_test_only:
+                    key = "last test"
+                elif use_harness:
+                    key = "all tests"
+                else:
+                    key = "no harness"
+                self._status.analysis[key] = reliability
                 # ensure same signature is always used
                 self._signature = replay.signature
 
@@ -400,7 +413,7 @@ class ReduceManager:
                 "* testcase was %s reliable with the harness",
                 "more" if harness_crashes > non_harness_crashes else "less",
             )
-        return (repeat, min_crashes, info)
+        return (repeat, min_crashes)
 
     def testcase_size(self):
         """Calculate the current testcase size.
@@ -410,33 +423,28 @@ class ReduceManager:
         """
         return sum(tc.data_size for tc in self.testcases)
 
-    def run(self, repeat=1, min_results=1):
+    def run(self, repeat=1, launch_attempts=3, min_results=1):
         """Run testcase reduction.
 
         Args:
             repeat (int): Maximum number of times to run the TestCase.
+            launch_attempts (int): Number of attempts to launch the browser.
             min_results (int): Minimum number of results needed before run can
                                be considered successful.
 
         Returns:
-            int: One of the `Session.EXIT_*` constants.
+            int: One of the Exit enum values.
         """
         any_success = False
         sig_given = self._signature is not None
-        last_reports = None
         last_tried = None
-        self._stats.add("init", self.testcase_size())
+        self._status.record("init")
         # record total stats overall so that any time missed by individual milestones
         # will still be included in the total
-        with self._stats.add_timed("final", self.testcase_size) as total_stats:
+        with self._status.measure("final"):
             if self._use_analysis:
-                with total_stats.add_timed("analysis") as stats:
-                    (
-                        repeat,
-                        min_results,
-                        reliability_info,
-                    ) = self.run_reliability_analysis(stats)
-                self._stats.add_info("Analysis", reliability_info)
+                with self._status.measure("analysis"):
+                    (repeat, min_results) = self.run_reliability_analysis()
                 any_success = True  # analysis ran and didn't raise
             # multi part test cases should always use relaunch == 1
             # since that can mean a delay is required
@@ -450,22 +458,18 @@ class ReduceManager:
                 min_results,
                 relaunch,
             )
-            self._stats.add_info(
-                "Run Parameters",
-                {
-                    "Repeat": repeat,
-                    "Min Crashes": min_results,
-                    "Relaunch": relaunch,
-                    "Harness": self._use_harness,
-                },
-            )
+            self._status.run_params["harness"] = self._use_harness
+            self._status.run_params["min crashes"] = min_results
+            self._status.run_params["relaunch"] = relaunch
+            self._status.run_params["repeat"] = repeat
 
-            for strategy_no, strategy in enumerate(self.strategies):
+            for strategy_no, strategy in enumerate(self.strategies, start=1):
+                self._status.current_strategy_idx = strategy_no
                 LOG.info("")
                 LOG.info(
                     "Using strategy %s (%d/%d)",
                     strategy,
-                    strategy_no + 1,
+                    strategy_no,
                     len(self.strategies),
                 )
                 replay = ReplayManager(
@@ -483,10 +487,12 @@ class ReduceManager:
                     last_tried = None
 
                 strategy_last_report = time()
-                strategy_stats = total_stats.add_timed(strategy.name)
+                strategy_stats = self._status.measure(strategy.name)
                 best_results = []
+                other_results = {}
                 try:
                     with replay, strategy, strategy_stats:
+                        self._status.report(force=True)
                         for reduction in strategy:
                             keep_reduction = False
                             results = []
@@ -499,11 +505,12 @@ class ReduceManager:
                                     expect_hang=self._expect_hang,
                                     idle_delay=self._idle_delay,
                                     idle_threshold=self._idle_threshold,
+                                    launch_attempts=launch_attempts,
                                     min_results=min_results,
                                     repeat=repeat,
+                                    on_iteration_cb=self._on_replay_iteration,
                                 )
-                                strategy_stats.add_iterations(replay.status.iteration)
-                                strategy_stats.add_attempts(1)
+                                self._status.attempts += 1
                                 self.update_timeout(results)
                                 # get the first expected result (if any),
                                 #   and update the strategy
@@ -513,7 +520,7 @@ class ReduceManager:
                                 )
                                 success = first_expected is not None
                                 if success:
-                                    strategy_stats.add_successes(1)
+                                    self._status.successes += 1
                                     if (
                                         not self._any_crash
                                         and self._signature_desc is None
@@ -522,6 +529,7 @@ class ReduceManager:
                                             # pylint: disable=line-too-long
                                             first_expected.report.crash_info.createShortSignature()  # noqa: E501
                                         )
+                                self._status.report()
                                 served = None
                                 if success and not self._any_crash:
                                     served = first_expected.served
@@ -535,6 +543,16 @@ class ReduceManager:
                                     LOG.info("Reduction succeeded")
                                     for testcase in self.testcases:
                                         testcase.cleanup()
+                                    # add target assets to test cases
+                                    if not self.target.assets.is_empty():
+                                        for test in reduction:
+                                            test.assets = self.target.assets
+                                    # add target environment variables
+                                    if self.target.filtered_environ():
+                                        for test in reduction:
+                                            test.env_vars = (
+                                                self.target.filtered_environ()
+                                            )
                                     self.testcases = reduction
                                     keep_reduction = True
                                     # cleanup old best results
@@ -551,11 +569,50 @@ class ReduceManager:
                                     ]
                                 else:
                                     LOG.info("Attempt failed")
+
                                 # if the reduction found other crashes,
-                                #   report those immediately
-                                self.report(
-                                    results, reduction, self._stats.copy(strategy_stats)
-                                )
+                                # save those for reporting later
+
+                                # only save the smallest testcase that has found
+                                # each result
+                                for result in results:
+                                    other_result_exists = bool(
+                                        result.report.minor in other_results
+                                    )
+
+                                    is_smaller = None
+                                    if other_result_exists:
+                                        # we have a result already queued for this sig
+                                        # check size to see which to keep
+                                        reduction_size = sum(
+                                            tc.data_size for tc in reduction
+                                        )
+                                        _, old_reduction = other_results[
+                                            result.report.minor
+                                        ]
+                                        old_size = sum(
+                                            tc.data_size for tc in old_reduction
+                                        )
+                                        is_smaller = bool(reduction_size < old_size)
+
+                                    if not other_result_exists or is_smaller:
+                                        if other_result_exists:
+                                            # clean-up old result
+                                            old_result, old_reduction = other_results[
+                                                result.report.minor
+                                            ]
+                                            old_result.report.cleanup()
+                                            for testcase in old_reduction:
+                                                testcase.cleanup()
+                                        # store this reduction for later reporting
+                                        # as the other result
+                                        other_results[result.report.minor] = (
+                                            result,
+                                            [
+                                                testcase.clone()
+                                                for testcase in reduction
+                                            ],
+                                        )
 
                                 now = time()
                                 if (
@@ -564,10 +621,9 @@ class ReduceManager:
                                     and now - strategy_last_report
                                     > self._report_periodically
                                 ):
-                                    last_reports = self.report(
+                                    self._status.last_reports = self.report(
                                         best_results,
                                         self.testcases,
-                                        self._stats.copy(strategy_stats),
                                     )
                                     for result in best_results:
                                         result.report.cleanup()
@@ -579,85 +635,81 @@ class ReduceManager:
                                 if not keep_reduction:
                                     for testcase in reduction:
                                         testcase.cleanup()
-                                for result in results:
-                                    result.report.cleanup()
 
                         # if self._signature was already set, this will do nothing
                         # otherwise, ensure the first found signature is used throughout
                         self._signature = replay.signature
 
                     if best_results:
-                        last_reports = self.report(
-                            best_results, self.testcases, self._stats.copy(total_stats)
+                        self._status.last_reports = self.report(
+                            best_results, self.testcases
                         )
+                    for result, reduction in other_results.values():
+                        self.report([result], reduction)
 
                 except KeyboardInterrupt:
                     if best_results:
-                        last_reports = self.report(
-                            best_results, self.testcases, self._stats.copy(total_stats)
+                        self._status.last_reports = self.report(
+                            best_results, self.testcases
                         )
                         LOG.warning(
-                            "Ctrl+C detected, best reduction so far " "reported as %r",
-                            last_reports,
+                            "Ctrl+C detected, best reduction so far reported as %r",
+                            self._status.last_reports,
                         )
                     raise
                 finally:
                     for result in best_results:
                         result.report.cleanup()
+                    for result, reduction in other_results.values():
+                        result.report.cleanup()
+                        for testcase in reduction:
+                            testcase.cleanup()
 
                 # store "tried" cache to pass to next strategy
                 last_tried = strategy.get_tried()
 
             # if we complete all strategies, mark the last reported crashes as reduced
-            if self._report_to_fuzzmanager and last_reports:
-                for crash_id in last_reports:
+            if self._report_to_fuzzmanager and self._status.last_reports:
+                for crash_id in self._status.last_reports:
                     LOG.info(
-                        "Updating crash %d to quality %s",
+                        "Updating crash %d to %s (Q%d)",
                         crash_id,
-                        FuzzManagerReporter.quality_name(
-                            FuzzManagerReporter.QUAL_REDUCED_RESULT
-                        ),
+                        Quality.REDUCED.name,
+                        Quality.REDUCED,
                     )
-                    CrashEntry(
-                        crash_id
-                    ).testcase_quality = FuzzManagerReporter.QUAL_REDUCED_RESULT
+                    CrashEntry(crash_id).testcase_quality = Quality.REDUCED.value
 
         # it's possible we made it this far without ever setting signature_desc.
         # this is only possible if --no-analysis is given
         # just give None instead of trying to format the CrashSignature
-        self._stats.add_info(
-            "Signature",
-            {
-                "given": sig_given,
-                "any": self._any_crash,
-                "description": self._signature_desc,
-            },
-        )
+        self._status.signature_info["any"] = self._any_crash
+        self._status.signature_info["description"] = self._signature_desc
+        self._status.signature_info["given"] = sig_given
 
         # log a summary of what was done.
         LOG.info(
             "Reduction summary:%s%s",
             os.linesep,
-            os.linesep.join(self._stats.format_lines()),
+            ReductionStatusReporter([self._status]).summary(),
         )
+        self._status.report(force=True)
 
         if any_success:
-            return Session.EXIT_SUCCESS
-        return Session.EXIT_FAILURE
+            return Exit.SUCCESS
+        return Exit.FAILURE
 
-    def report(self, results, testcases, stats=None):
+    def report(self, results, testcases):
         """Report results, either to FuzzManager or to filesystem.
 
         Arguments:
             results (list(ReplayResult)): Results observed during reduction.
             testcases (list(TestCase)): Testcases used to trigger results.
-            stats (ReductionStats): Statistics for reduction of these results.
-                                    (may be different than `self._stats`)
 
         Returns:
             list(*): List of return values from `reporter.submit()`.
         """
         ret_values = []
+        status = self._status.copy()  # copy implicitly closes open counters
         for result in results:
             if self._report_to_fuzzmanager:
                 reporter = FuzzManagerReporter(self._report_tool)
@@ -666,17 +718,15 @@ class ReduceManager:
             else:
                 report_dir = "reports" if result.expected else "other_reports"
                 reporter = FilesystemReporter(
-                    report_path=str(self._log_path / report_dir), major_bucket=False
+                    report_path=self._log_path / report_dir, major_bucket=False
                 )
-            # write reduction stats
-            if stats is not None:
-                report_path = Path(result.report.path)
-                with (report_path / "reduce_stats.txt").open("w") as out:
-                    for line in stats.format_lines():
-                        print(line, file=out)
-                (report_path / "reduce_stats.json").write_text(stats.json())
-                if self._report_to_fuzzmanager:
-                    stats.add_to_reporter(reporter)
+            # write reduction stats for expected results
+            if result.expected:
+                (Path(result.report.path) / "reduce_stats.txt").write_text(
+                    ReductionStatusReporter([status]).summary()
+                )
+            if self._report_to_fuzzmanager:
+                status.add_to_reporter(reporter, expected=result.expected)
             # clone the tests so we can safely call purge_optional here for each report
             # (report.served may be different for non-expected or any-crash results)
             clones = [test.clone() for test in testcases]
@@ -684,7 +734,11 @@ class ReduceManager:
                 if result.served is not None:
                     for clone, served in zip(clones, result.served):
                         clone.purge_optional(served)
-                ret_values.append(reporter.submit(clones, report=result.report))
+                result = reporter.submit(clones, result.report)
+                if result is not None:
+                    if isinstance(result, Path):
+                        result = str(result)
+                    ret_values.append(result)
             finally:
                 for clone in clones:
                     clone.cleanup()
@@ -708,10 +762,10 @@ class ReduceManager:
 
         LOG.info("Starting Grizzly Reduce")
 
+        if args.headless:
+            LOG.info("Running browser headless (%s)", args.headless)
         if args.ignore:
             LOG.info("Ignoring: %s", ", ".join(args.ignore))
-        if args.xvfb:
-            LOG.info("Running with Xvfb")
         if args.pernosco:
             LOG.info("Running with RR (Pernosco mode)")
         elif args.rr:
@@ -719,11 +773,11 @@ class ReduceManager:
         elif args.valgrind:
             LOG.info("Running with Valgrind. This will be SLOW!")
 
+        assets = None
         signature = None
         signature_desc = None
         target = None
         testcases = []
-        tmp_prefs = None
         try:
             if args.sig:
                 signature = CrashSignature.fromFile(args.sig)
@@ -733,12 +787,12 @@ class ReduceManager:
                     signature_desc = meta["shortDescription"]
 
             try:
-                testcases = ReplayManager.load_testcases(
-                    args.input, args.prefs is None, subset=args.test_index
+                testcases, assets, env_vars = ReplayManager.load_testcases(
+                    str(args.input), subset=args.test_index
                 )
             except TestCaseLoadFailure as exc:
                 LOG.error("Error: %s", str(exc))
-                return Session.EXIT_ERROR
+                return Exit.ERROR
 
             if args.tool is None and testcases[0].adapter_name is not None:
                 LOG.warning(
@@ -755,7 +809,7 @@ class ReduceManager:
                         "Error: '--no-harness' cannot be used with multiple "
                         "testcases. Perhaps '--test-index' can help."
                     )
-                    return Session.EXIT_ARGS
+                    return Exit.ARGS
                 LOG.debug("--no-harness specified relaunch set to 1")
                 args.relaunch = 1
 
@@ -770,33 +824,24 @@ class ReduceManager:
             LOG.debug("initializing the Target")
             target = load_plugin(args.platform, "grizzly_targets", Target)(
                 args.binary,
-                args.extension,
                 args.launch_timeout,
                 args.log_limit,
                 args.memory,
+                assets=assets,
+                headless=args.headless,
                 pernosco=args.pernosco,
                 rr=args.rr,
                 valgrind=args.valgrind,
-                xvfb=args.xvfb,
             )
-            # prioritize specified prefs.js file over included file
-            if args.prefs is not None:
-                for testcase in testcases:
-                    testcase.add_meta(TestFile.from_file(args.prefs, "prefs.js"))
-                LOG.info("Using specified prefs.js")
-                target.prefs = args.prefs
-            else:
-                for testcase in testcases:
-                    prefs_tf = testcase.get_file("prefs.js")
-                    if prefs_tf:
-                        tmp_prefs = Path(
-                            mkdtemp(prefix="prefs_", dir=grz_tmp("replay"))
-                        )
-                        prefs_tf.dump(str(tmp_prefs))
-                        LOG.info("Using prefs.js from testcase")
-                        target.prefs = str(tmp_prefs / "prefs.js")
-                        break
-
+            # local environ takes priority over environ loaded from test case
+            if env_vars is not None:
+                env_vars.update(target.environ)
+                target.environ = env_vars
+                env_vars = None
+            # TODO: support overriding existing assets
+            # prioritize specified assets over included
+            target.assets.add_batch(args.asset)
+            target.process_assets()
             LOG.debug("starting sapphire server")
             # launch HTTP server used to serve test cases
             with Sapphire(auto_close=1, timeout=timeout) as server:
@@ -812,6 +857,7 @@ class ReduceManager:
                     expect_hang=expect_hang,
                     idle_delay=args.idle_delay,
                     idle_threshold=args.idle_threshold,
+                    reducer_crash_id=args.original_crash_id,
                     relaunch=relaunch,
                     report_period=args.report_period,
                     report_to_fuzzmanager=args.fuzzmanager,
@@ -822,7 +868,11 @@ class ReduceManager:
                     use_analysis=not args.no_analysis,
                     use_harness=not args.no_harness,
                 )
-                return_code = mgr.run(repeat=args.repeat, min_results=args.min_crashes)
+                return_code = mgr.run(
+                    repeat=args.repeat,
+                    launch_attempts=args.launch_attempts,
+                    min_results=args.min_crashes,
+                )
             return return_code
 
         except ConfigError as exc:
@@ -831,7 +881,7 @@ class ReduceManager:
 
         except KeyboardInterrupt as exc:
             LOG.error("Exception: %r", exc)
-            return Session.EXIT_ABORT
+            return Exit.ABORT
 
         except (TargetLaunchError, TargetLaunchTimeout) as exc:
             LOG.error("Exception: %s", exc)
@@ -840,7 +890,7 @@ class ReduceManager:
                 LOG.error("Logs can be found here %r", path)
                 reporter = FilesystemReporter(path, major_bucket=False)
                 reporter.submit([], exc.report)
-            return Session.EXIT_LAUNCH_FAILURE
+            return Exit.LAUNCH_FAILURE
 
         except GrizzlyReduceBaseException as exc:
             LOG.error(exc.msg)
@@ -848,7 +898,7 @@ class ReduceManager:
 
         except Exception:  # pylint: disable=broad-except
             LOG.exception("Exception during reduction!")
-            return Session.EXIT_ERROR
+            return Exit.ERROR
 
         finally:
             LOG.info("Shutting down...")
@@ -856,6 +906,6 @@ class ReduceManager:
                 target.cleanup()
             for testcase in testcases:
                 testcase.cleanup()
-            if tmp_prefs is not None:
-                rmtree(str(tmp_prefs), ignore_errors=True)
+            if assets:
+                assets.cleanup()
             LOG.info("Done.")
