@@ -3,13 +3,13 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 """Grizzly Reduction exceptions."""
-from ..session import Session
+from ..common.utils import Exit
 
 
 class GrizzlyReduceBaseException(Exception):
     """Base for other Grizzly Reducer specific exceptions."""
 
-    def __init__(self, msg, code=Session.EXIT_ERROR):
+    def __init__(self, msg, code=Exit.ERROR):
         super().__init__()
         self.msg = msg
         self.code = code
@@ -19,4 +19,4 @@ class NotReproducible(GrizzlyReduceBaseException):
     """Crash was not observed when expected during reduction."""
 
     def __init__(self, msg):
-        super().__init__(msg, code=Session.EXIT_FAILURE)
+        super().__init__(msg, code=Exit.FAILURE)
