@@ -10,21 +10,24 @@ from functools import partial, wraps
 from itertools import count
 from logging import getLogger
 
-import pytest
-from pytest import raises
+from pytest import mark, raises
 
 from sapphire import Sapphire
 
-from ..common import Report, TestCase
+from ..common.reporter import Quality, Report
+from ..common.storage import TestCase
 from ..replay import ReplayResult
-from ..target import Target
+from ..target import AssetManager, Target
 from . import ReduceManager
 from .exceptions import NotReproducible
 from .strategies import Strategy
 
 LOG = getLogger(__name__)
-pytestmark = pytest.mark.usefixtures(
-    "tmp_path_fm_config", "reporter_sequential_strftime"
+pytestmark = mark.usefixtures(
+    "reporter_sequential_strftime",
+    "tmp_path_fm_config",
+    "tmp_path_replay_status_db",
+    "tmp_path_reduce_status_db",
 )
 
 
@@ -59,7 +62,7 @@ AnalysisTestParams = namedtuple(
 )
 
 
-@pytest.mark.parametrize(
+@mark.parametrize(
     AnalysisTestParams._fields,
     [
         # perfect with harness
@@ -242,7 +245,6 @@ def test_analysis(
     """test that analysis sets reasonable params"""
     replayer = mocker.patch("grizzly.reduce.core.ReplayManager", autospec=True)
     replayer = replayer.return_value.__enter__.return_value
-    replayer.status.iteration = 11
     crashes = []
     expected_iters = 0
     if harness_last_crashes is not None:
@@ -261,6 +263,7 @@ def test_analysis(
         assert repeat <= len(crashes)
         assert not kw["exit_early"]
         for _ in range(repeat):
+            kw["on_iteration_cb"]()
             LOG.debug("interesting: %r", crashes[0])
             if crashes.pop(0):
                 log_path = tmp_path / ("crash%d_logs" % (replayer.run.call_count,))
@@ -273,37 +276,25 @@ def test_analysis(
     replayer.run.side_effect = replay_run
 
     test = TestCase("test.html", None, "test-adapter")
-    test.add_from_data("1", "test.html")
+    test.add_from_bytes(b"1", "test.html")
     tests = [test]
     if harness_last_crashes is not None:
         test = TestCase("test.html", None, "test-adapter")
-        test.add_from_data("2", "test.html")
+        test.add_from_bytes(b"2", "test.html")
         tests.append(test.clone())
     log_path = tmp_path / "logs"
 
-    class _ReduceStats:
-        def __init__(self):
-            self._iters = 0
-
-        def add_iterations(self, iters):
-            self._iters += iters
-
-        @staticmethod
-        def _stop_early(_):
-            pass
-
-    stats = _ReduceStats()
     try:
         mgr = ReduceManager(
             None,
-            mocker.Mock(spec=Sapphire, timeout=30),
-            mocker.Mock(spec=Target),
+            mocker.Mock(spec_set=Sapphire, timeout=30),
+            mocker.Mock(spec_set=Target),
             tests,
             None,
             log_path,
             use_harness=use_harness,
         )
-        repeat, min_crashes, _ = mgr.run_reliability_analysis(stats)
+        repeat, min_crashes = mgr.run_reliability_analysis()
     finally:
         for test in tests:
             test.cleanup()
@@ -312,7 +303,7 @@ def test_analysis(
     assert repeat == expected_repeat
     assert min_crashes == expected_min_crashes
     assert mgr._use_harness == result_harness
-    assert stats._iters == expected_iters * 11
+    assert mgr._status.iterations == expected_iters * 11
 
 
 def _ignore_arg(func):
@@ -332,12 +323,12 @@ ReproTestParams = namedtuple(
 )
 
 
-@pytest.mark.parametrize(
+@mark.parametrize(
     ReproTestParams._fields,
     [
         # no repro
         ReproTestParams(
-            original="123456\n",
+            original=b"123456\n",
             strategies=["check"],
             detect_failure=lambda _: False,
             interesting_str="%r",
@@ -351,7 +342,7 @@ ReproTestParams = namedtuple(
         ),
         # reproduces, one strategy, no reduction works
         ReproTestParams(
-            original="1\n2\n3\n",
+            original=b"1\n2\n3\n",
             strategies=["check", "lines"],
             detect_failure=lambda contents: contents == "1\n2\n3\n",
             interesting_str="%r == '1\n2\n3\n'",
@@ -365,7 +356,7 @@ ReproTestParams = namedtuple(
         ),
         # reproduces, one strategy, some reduction works
         ReproTestParams(
-            original="odd\neven\n" * 3,
+            original=b"odd\neven\n" * 3,
             strategies=["check", "lines"],
             detect_failure=lambda contents: sum(
                 1 for line in contents.splitlines() if line == "odd"
@@ -382,7 +373,7 @@ ReproTestParams = namedtuple(
         ),
         # reproduces, one strategy, reduction only finds other
         ReproTestParams(
-            original="1\n2\n3\n",
+            original=b"1\n2\n3\n",
             strategies=["check", "lines"],
             detect_failure=bool,
             interesting_str="%r != ''",
@@ -390,13 +381,13 @@ ReproTestParams = namedtuple(
             expected_run_calls=6,
             n_reports=1,
             reports={"1\n2\n3\n"},
-            n_other=5,
-            other_reports={"1\n", "3\n", "1\n2\n", "1\n3\n", "2\n3\n"},
+            n_other=1,
+            other_reports={"1\n"},
             result=0,
         ),
         # reproduces, 50% iterations work, 1 iteration produces other sig
         ReproTestParams(
-            original="1\n2\n3\n4\n5\n6\n",
+            original=b"1\n2\n3\n4\n5\n6\n",
             strategies=["check", "lines"],
             detect_failure=lambda contents: set(contents.splitlines()) >= set("135"),
             interesting_str="%r contains {'1', '3', '5'}",
@@ -410,7 +401,7 @@ ReproTestParams = namedtuple(
         ),
         # reproduces, two strategies, 1st no reduction, 2nd 50% reduction
         ReproTestParams(
-            original="A1\nA2\nA3\nA4\nA5\nA6\n",
+            original=b"A1\nA2\nA3\nA4\nA5\nA6\n",
             strategies=["check", "lines", "chars"],
             detect_failure=(
                 lambda contents: (
@@ -430,7 +421,7 @@ ReproTestParams = namedtuple(
         ),
         # reproduces, two strategies, 1st 50% reduction, 2nd no reduction
         ReproTestParams(
-            original="A1\nA2\nA3\nA4\nA5\nA6\n",
+            original=b"A1\nA2\nA3\nA4\nA5\nA6\n",
             strategies=["check", "lines", "chars"],
             detect_failure=(
                 lambda contents: (
@@ -449,7 +440,7 @@ ReproTestParams = namedtuple(
         ),
         # reproduces, two strategies, reduce only produces other sig
         ReproTestParams(
-            original="1\n2\n3\n",
+            original=b"1\n2\n3\n",
             strategies=["check", "lines", "chars"],
             detect_failure=bool,
             interesting_str="%r != ''",
@@ -457,25 +448,15 @@ ReproTestParams = namedtuple(
             expected_run_calls=16,
             n_reports=1,
             reports={"1\n2\n3\n"},
-            n_other=15,
+            n_other=2,
             other_reports={
-                "\n2\n3\n",
                 "1\n",
-                "1\n\n3\n",
-                "1\n2\n",
-                "1\n2\n\n",
-                "1\n2\n3",
-                "1\n23\n",
-                "1\n3\n",
-                "12\n3\n",
-                "2\n3\n",
-                "3\n",
             },
             result=0,
         ),
         # reproduces, one strategy, testcase reduces to 0
         ReproTestParams(
-            original="1\n2\n3\n",
+            original=b"1\n2\n3\n",
             strategies=["check", "lines"],
             detect_failure=lambda _: True,
             interesting_str="%r is anything, incl. empty",
@@ -489,7 +470,7 @@ ReproTestParams = namedtuple(
         ),
         # reproduces, two strategies, 1st no reduce, 2nd testcase reduces to 0
         ReproTestParams(
-            original="1\n2\n3\n",
+            original=b"1\n2\n3\n",
             strategies=["check", "lines", "lines"],
             detect_failure=_ignore_arg(
                 partial([True, False, False, False, False, False, True, True].pop, 0)
@@ -505,7 +486,7 @@ ReproTestParams = namedtuple(
         ),
     ],
 )
-@pytest.mark.usefixtures("reporter_sequential_strftime")
+@mark.usefixtures("reporter_sequential_strftime")
 def test_repro(
     mocker,
     tmp_path,
@@ -526,11 +507,11 @@ def test_repro(
     mocker.patch("grizzly.reduce.strategies.lithium._contains_dd", return_value=True)
     replayer = mocker.patch("grizzly.reduce.core.ReplayManager", autospec=True)
     replayer = replayer.return_value
-    replayer.status.iteration = 1
 
-    def replay_run(testcases, _time_limit, **_):
+    def replay_run(testcases, _time_limit, **kw):
+        kw["on_iteration_cb"]()
         for test in testcases:
-            contents = test.get_file("test.html").data.decode("ascii")
+            contents = test.get_file("test.html").data_file.read_text()
             # pylint: disable=logging-not-lazy
             LOG.debug("interesting if " + interesting_str, contents)
             if detect_failure(contents):
@@ -548,16 +529,17 @@ def test_repro(
     replayer.run.side_effect = replay_run
 
     test = TestCase("test.html", None, "test-adapter")
-    test.add_from_data(original, "test.html")
+    test.add_from_bytes(original, "test.html")
     tests = [test]
     log_path = tmp_path / "logs"
 
-    target = mocker.Mock(spec=Target)
-    target.relaunch = 1
+    target = mocker.Mock(spec_set=Target)
+    target.filtered_environ.return_value = dict()
+    target.assets = mocker.Mock(spec_set=AssetManager)
     try:
         mgr = ReduceManager(
             [],
-            mocker.Mock(spec=Sapphire, timeout=30),
+            mocker.Mock(spec_set=Sapphire, timeout=30),
             target,
             tests,
             strategies,
@@ -603,9 +585,9 @@ def test_report_01(mocker, tmp_path):
 
     replayer = mocker.patch("grizzly.reduce.core.ReplayManager", autospec=True)
     replayer = replayer.return_value
-    replayer.status.iteration = 1
 
-    def replay_run(_tests, _time_limit, **_kw):
+    def replay_run(_tests, _time_limit, **kw):
+        kw["on_iteration_cb"]()
         if replayer.run.call_count in {20, 40}:
             log_path = tmp_path / ("crash%d_logs" % (replayer.run.call_count,))
             log_path.mkdir()
@@ -617,30 +599,31 @@ def test_report_01(mocker, tmp_path):
     replayer.run.side_effect = replay_run
 
     (tmp_path / "test.html").touch()
-    testcases = TestCase.load(str(tmp_path / "test.html"), False)
+    testcases = TestCase.load(str(tmp_path / "test.html"))
     assert testcases
     log_path = tmp_path / "logs"
 
-    fake_strat = mocker.MagicMock(spec=Strategy)
+    fake_strat = mocker.MagicMock(spec_set=Strategy)
     fake_strat.return_value.name = "fake"
 
     def fake_iter():
         for count_ in range(1, 61):
             LOG.debug("fake_iter() %d", count_)
             (tmp_path / "test.html").write_text(str(count_))
-            testcases = TestCase.load(str(tmp_path / "test.html"), False)
+            testcases = TestCase.load(str(tmp_path / "test.html"))
             assert testcases
             yield testcases
 
     fake_strat.return_value.__iter__.side_effect = fake_iter
     mocker.patch("grizzly.reduce.core.STRATEGIES", new={"fake": fake_strat})
 
-    target = mocker.Mock(spec=Target)
-    target.relaunch = 1
+    target = mocker.Mock(spec_set=Target)
+    target.filtered_environ.return_value = dict()
+    target.assets = mocker.Mock(spec_set=AssetManager)
     try:
         mgr = ReduceManager(
             [],
-            mocker.Mock(spec=Sapphire, timeout=30),
+            mocker.Mock(spec_set=Sapphire, timeout=30),
             target,
             testcases,
             ["fake"],
@@ -673,9 +656,9 @@ def test_report_02(mocker, tmp_path):
 
     replayer = mocker.patch("grizzly.reduce.core.ReplayManager", autospec=True)
     replayer = replayer.return_value
-    replayer.status.iteration = 1
 
-    def replay_run(_tests, _time_limit, **_kw):
+    def replay_run(_tests, _time_limit, **kw):
+        kw["on_iteration_cb"]()
         if replayer.run.call_count in {10, 20}:
             log_path = tmp_path / ("crash%d_logs" % (replayer.run.call_count,))
             log_path.mkdir()
@@ -687,18 +670,18 @@ def test_report_02(mocker, tmp_path):
     replayer.run.side_effect = replay_run
 
     (tmp_path / "test.html").touch()
-    testcases = TestCase.load(str(tmp_path / "test.html"), False)
+    testcases = TestCase.load(str(tmp_path / "test.html"))
     assert testcases
     log_path = tmp_path / "logs"
 
-    fake_strat = mocker.MagicMock(spec=Strategy)
+    fake_strat = mocker.MagicMock(spec_set=Strategy)
     fake_strat.return_value.name = "fake"
 
     def fake_iter():
         for count_ in range(1, 31):
             LOG.debug("fake_iter() %d", count_)
             (tmp_path / "test.html").write_text(str(count_))
-            testcases = TestCase.load(str(tmp_path / "test.html"), False)
+            testcases = TestCase.load(str(tmp_path / "test.html"))
             assert testcases
             yield testcases
         raise KeyboardInterrupt()
@@ -706,12 +689,13 @@ def test_report_02(mocker, tmp_path):
     fake_strat.return_value.__iter__.side_effect = fake_iter
     mocker.patch("grizzly.reduce.core.STRATEGIES", new={"fake": fake_strat})
 
-    target = mocker.Mock(spec=Target)
-    target.relaunch = 1
+    target = mocker.Mock(spec_set=Target)
+    target.filtered_environ.return_value = dict()
+    target.assets = mocker.Mock(spec_set=AssetManager)
     try:
         mgr = ReduceManager(
             [],
-            mocker.Mock(spec=Sapphire, timeout=30),
+            mocker.Mock(spec_set=Sapphire, timeout=30),
             target,
             testcases,
             ["fake"],
@@ -737,15 +721,15 @@ def test_report_02(mocker, tmp_path):
 
 
 def test_quality_update(mocker, tmp_path):
-    """test that the final result gets changed to Q0 with --fuzzmanager"""
+    """test that the final result gets changed to REDUCED with --fuzzmanager"""
     mocker.patch("grizzly.reduce.strategies.lithium._contains_dd", return_value=True)
     replayer = mocker.patch("grizzly.reduce.core.ReplayManager", autospec=True)
     replayer = replayer.return_value
-    replayer.status.iteration = 1
 
-    def replay_run(testcases, _time_limit, **_kw):
+    def replay_run(testcases, _time_limit, **kw):
+        kw["on_iteration_cb"]()
         for test in testcases:
-            contents = test.get_file("test.html").data.decode("ascii")
+            contents = test.get_file("test.html").data_file.read_text()
             if not contents.strip():
                 continue
             log_path = tmp_path / ("crash%d_logs" % (replayer.run.call_count,))
@@ -758,20 +742,21 @@ def test_quality_update(mocker, tmp_path):
     replayer.run.side_effect = replay_run
 
     (tmp_path / "test.html").write_text("123\n")
-    testcases = TestCase.load(str(tmp_path / "test.html"), False)
+    testcases = TestCase.load(str(tmp_path / "test.html"))
     assert testcases
     log_path = tmp_path / "logs"
 
-    mocker.patch("grizzly.common.reporter.Collector")
-    reporter = mocker.patch("grizzly.reduce.core.FuzzManagerReporter")
-    reporter.QUAL_REDUCED_RESULT = 0
+    mocker.patch("grizzly.common.reporter.Collector", autospec=True)
+    reporter = mocker.patch("grizzly.reduce.core.FuzzManagerReporter", autospec=True)
+    reporter.return_value.submit.return_value = 1234
     update_coll = mocker.patch("grizzly.common.fuzzmanager.Collector")
-    target = mocker.Mock(spec=Target)
-    target.relaunch = 1
+    target = mocker.Mock(spec_set=Target)
+    target.filtered_environ.return_value = dict()
+    target.assets = mocker.Mock(spec_set=AssetManager)
     try:
         mgr = ReduceManager(
             [],
-            mocker.Mock(spec=Sapphire, timeout=30),
+            mocker.Mock(spec_set=Sapphire, timeout=30),
             target,
             testcases,
             ["check", "lines"],
@@ -785,19 +770,81 @@ def test_quality_update(mocker, tmp_path):
             test.cleanup()
 
     assert reporter.return_value.submit.call_count == 1
-    report_args, report_kwds = reporter.return_value.submit.call_args
-    assert len(report_args) == 1
+    report_args, _ = reporter.return_value.submit.call_args
+    assert len(report_args) == 2
     assert isinstance(report_args[0], list)
     assert len(report_args[0]) == 1
     assert isinstance(report_args[0][0], TestCase)
-    assert report_kwds.keys() == {"report"}
+    assert isinstance(report_args[1], Report)
     assert update_coll.call_count == 1
     assert update_coll.return_value.patch.call_count == 1
     assert update_coll.return_value.patch.call_args[1] == {
-        "data": {"testcase_quality": reporter.QUAL_REDUCED_RESULT},
+        "data": {"testcase_quality": Quality.REDUCED},
     }
 
 
+def test_include_assets_and_environ(mocker, tmp_path):
+    """test report with assets and environment variables"""
+    mocker.patch("grizzly.reduce.strategies.lithium._contains_dd", return_value=True)
+    replayer = mocker.patch("grizzly.reduce.core.ReplayManager", autospec=True)
+    replayer = replayer.return_value
+
+    def replay_run(testcases, _time_limit, **kw):
+        kw["on_iteration_cb"]()
+        for test in testcases:
+            contents = test.get_file("test.html").data_file.read_text()
+            if not contents.strip():
+                continue
+            log_path = tmp_path / ("crash%d_logs" % (replayer.run.call_count,))
+            log_path.mkdir()
+            _fake_save_logs_foo(log_path)
+            report = Report(str(log_path), "bin")
+            return [ReplayResult(report, [["test.html"]], [], True)]
+        return []
+
+    replayer.run.side_effect = replay_run
+
+    (tmp_path / "test.html").write_text("123\n")
+    testcases = TestCase.load(str(tmp_path / "test.html"))
+    assert testcases
+    log_path = tmp_path / "logs"
+
+    reporter = mocker.patch("grizzly.reduce.core.FilesystemReporter", autospec=True)
+
+    def submit(test_cases, report):
+        assert test_cases
+        assert isinstance(report, Report)
+        for test in test_cases:
+            assert test.assets.get("example")
+            assert test.env_vars == {"test": "abc"}
+            test.cleanup()
+
+    reporter.return_value.submit.side_effect = submit
+
+    target = mocker.Mock(spec_set=Target)
+    target.filtered_environ.return_value = {"test": "abc"}
+    with AssetManager(base_path=str(tmp_path)) as assets:
+        (tmp_path / "example_asset").touch()
+        assets.add("example", str(tmp_path / "example_asset"), copy=False)
+        target.assets = assets
+        try:
+            mgr = ReduceManager(
+                [],
+                mocker.Mock(spec_set=Sapphire, timeout=30),
+                target,
+                testcases,
+                ["check", "lines"],
+                log_path,
+                use_analysis=False,
+            )
+            assert mgr.run() == 0
+        finally:
+            for test in testcases:
+                test.cleanup()
+
+    assert reporter.return_value.submit.call_count == 1
+
+
 TimeoutTestParams = namedtuple(
     "TimeoutTestParams",
     "durations, interesting, static_timeout, idle_input, idle_output, iter_input,"
@@ -805,7 +852,7 @@ TimeoutTestParams = namedtuple(
 )
 
 
-@pytest.mark.parametrize(
+@mark.parametrize(
     TimeoutTestParams._fields,
     [
         # 0 duration test sets both timeouts to minimum
@@ -881,9 +928,9 @@ def test_timeout_update(
     mocker.patch("grizzly.reduce.strategies.lithium._contains_dd", return_value=True)
     replayer = mocker.patch("grizzly.reduce.core.ReplayManager", autospec=True)
     replayer = replayer.return_value
-    replayer.status.iteration = 1
 
-    def replay_run(_testcases, _time_limit, **_):
+    def replay_run(_testcases, _time_limit, **kw):
+        kw["on_iteration_cb"]()
         LOG.debug("interesting true")
         log_path = tmp_path / ("crash%d_logs" % (replayer.run.call_count,))
         log_path.mkdir()
@@ -894,14 +941,14 @@ def test_timeout_update(
     replayer.run.side_effect = replay_run
 
     test = TestCase("test.html", None, "test-adapter")
-    test.add_from_data("123\n", "test.html")
+    test.add_from_bytes(b"123\n", "test.html")
     tests = [test]
     log_path = tmp_path / "logs"
 
-    target = mocker.Mock(spec=Target)
-    target.relaunch = 1
-    server = mocker.Mock(spec=Sapphire)
-    server.timeout = iter_input
+    target = mocker.Mock(spec_set=Target)
+    target.filtered_environ.return_value = dict()
+    target.assets = mocker.Mock(spec_set=AssetManager)
+    server = mocker.Mock(spec_set=Sapphire, timeout=iter_input)
     try:
         mgr = ReduceManager(
             [],
