@@ -2,7 +2,8 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from os.path import exists, isfile
+from argparse import SUPPRESS
+from pathlib import Path
 
 from ..args import CommonArgs
 
@@ -10,8 +11,10 @@ from ..args import CommonArgs
 class ReplayArgs(CommonArgs):
     def __init__(self):
         super().__init__()
+        self.parser.set_defaults(logs=None)
         self.parser.add_argument(
             "input",
+            type=Path,
             help="Accepted input includes: "
             "1) A directory containing testcase data. "
             "2) A directory with one or more subdirectories containing testcase data. "
@@ -41,12 +44,6 @@ class ReplayArgs(CommonArgs):
             default=0,
             help="CPU usage threshold to mark the process as idle (default: disabled)",
         )
-        replay_args.add_argument(
-            "-l",
-            "--logs",
-            help="Location to save logs. If the path exists it must be empty, if it "
-            "does not exist it will be created.",
-        )
         replay_args.add_argument(
             "--min-crashes",
             type=int,
@@ -59,6 +56,8 @@ class ReplayArgs(CommonArgs):
             action="store_true",
             help="Don't use the harness for redirection. Implies '--relaunch=1'.",
         )
+        # hidden argument to add original crash ID as metadata when reported
+        replay_args.add_argument("--original-crash-id", type=int, help=SUPPRESS)
         replay_args.add_argument(
             "--repeat",
             type=int,
@@ -66,7 +65,9 @@ class ReplayArgs(CommonArgs):
             help="Run the testcase n times."
             " Helpful for intermittent testcases (default: %(default)sx)",
         )
-        replay_args.add_argument("--sig", help="Signature (JSON) file to match.")
+        replay_args.add_argument(
+            "--sig", type=Path, help="Signature (JSON) file to match."
+        )
         replay_args.add_argument(
             "--test-index",
             type=int,
@@ -76,6 +77,14 @@ class ReplayArgs(CommonArgs):
             "0 == Oldest, n-1 == Newest (default: run all testcases)",
         )
 
+        self.launcher_grp.add_argument(
+            "--post-launch-delay",
+            type=int,
+            default=None,
+            help="Number of seconds to wait after launching browser before "
+            " running the test. This provides an opportunity to connect debuggers. ",
+        )
+
         self.reporter_grp.add_argument(
             "--include-test",
             action="store_true",
@@ -91,14 +100,67 @@ class ReplayArgs(CommonArgs):
         if args.idle_threshold and args.idle_delay <= 0:
             self.parser.error("--idle-delay value must be positive")
 
-        if "input" not in self._sanity_skip and not exists(args.input):
-            self.parser.error("%r does not exist" % (args.input,))
+        if "input" not in self._sanity_skip and not args.input.exists():
+            self.parser.error("'%s' does not exist" % (args.input,))
+
+        if args.logs is None and (args.pernosco or args.rr):
+            self.parser.error("--logs must be set when using rr")
 
         if args.min_crashes < 1:
             self.parser.error("--min-crashes value must be positive")
 
+        if args.post_launch_delay is not None and args.post_launch_delay < 0:
+            self.parser.error("--post-launch-delay value must be positive")
+
         if args.repeat < 1:
             self.parser.error("--repeat value must be positive")
 
-        if args.sig is not None and not isfile(args.sig):
-            self.parser.error("signature file not found: %r" % (args.sig,))
+        if args.sig and not args.sig.is_file():
+            self.parser.error("signature file not found: '%s'" % (args.sig,))
+
+    def update_arg(self, name, new_type, help_msg):
+        # madhax alert!
+        #
+        # We need to modify the meaning of the 'input' positional to accept an int ID
+        # instead of a local testcase.
+        # This is not possible with the public argparse API.
+        #
+        # refs: stackoverflow.com/questions/32807319/disable-remove-argument-in-argparse
+        #       bugs.python.org/issue19462
+
+        # look up the action for the positional `input` arg
+        action = None
+        for arg in self.parser._actions:  # pylint: disable=protected-access
+            if arg.dest == name and not arg.option_strings:
+                action = arg
+                break
+        assert action is not None
+
+        # modify its type and help string
+        action.type = new_type
+        action.help = help_msg
+
+        # ... and Bob's your uncle
+        self._sanity_skip.add("input")
+
+
+class ReplayFuzzManagerIDArgs(ReplayArgs):
+    def __init__(self):
+        """Initialize argument parser."""
+        super().__init__()
+        self.update_arg("input", int, "FuzzManager ID to replay")
+
+
+class ReplayFuzzManagerIDQualityArgs(ReplayFuzzManagerIDArgs):
+    def __init__(self):
+        """Initialize argument parser."""
+        super().__init__()
+        self.parser.add_argument(
+            "--quality", type=int, help="Only try crashes with a given quality value"
+        )
+
+    def sanity_check(self, args):
+        super().sanity_check(args)
+
+        if args.quality is not None and args.quality < 0:
+            self.parser.error("'--quality' value cannot be negative")
