@@ -4,9 +4,7 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 from logging import getLogger
-from os.path import dirname
-from os.path import join as pathjoin
-from shutil import rmtree
+from pathlib import Path
 from tempfile import mkdtemp
 
 from FTB.Signatures.CrashInfo import CrashSignature
@@ -15,12 +13,11 @@ from sapphire import Sapphire, ServerMap
 
 from ..common.plugins import load as load_plugin
 from ..common.reporter import FilesystemReporter, FuzzManagerReporter, Report
-from ..common.runner import Runner, RunResult
+from ..common.runner import Runner
 from ..common.status import Status
-from ..common.storage import TestCase, TestCaseLoadFailure, TestFile
-from ..common.utils import TIMEOUT_DELAY, ConfigError, configure_logging, grz_tmp
-from ..session import Session
-from ..target import Target, TargetLaunchError, TargetLaunchTimeout
+from ..common.storage import TestCase, TestCaseLoadFailure
+from ..common.utils import TIMEOUT_DELAY, ConfigError, Exit, configure_logging, grz_tmp
+from ..target import Result, Target, TargetLaunchError, TargetLaunchTimeout
 
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith"]
@@ -40,8 +37,9 @@ class ReplayResult:
 
 
 class ReplayManager:
-    HARNESS_FILE = pathjoin(dirname(__file__), "..", "common", "harness.html")
+    HARNESS_FILE = str(Path(__file__).parent / ".." / "common" / "harness.html")
     DEFAULT_TIME_LIMIT = 30
+    STATUS_DB = str(Path(grz_tmp()) / "replay-status.db")
 
     __slots__ = (
         "ignore",
@@ -93,7 +91,8 @@ class ReplayManager:
     @staticmethod
     def check_match(signature, report, expect_hang):
         if signature is None:
-            return False
+            # Treat 'None' signature as a bucket if it's not a hang
+            return not report.is_hang
         if expect_hang and not report.is_hang:
             # avoid catching other crashes with forgiving hang signatures
             return False
@@ -108,8 +107,7 @@ class ReplayManager:
         Returns:
             None
         """
-        if self.status is not None:
-            self.status.cleanup()
+        # TODO: Do we need this anymore?
 
     @staticmethod
     def expect_hang(ignore, signature, tests):
@@ -128,31 +126,45 @@ class ReplayManager:
         is_hang = any(x.hang for x in tests)
         if is_hang:
             if signature is None:
-                raise ConfigError(
-                    "Hangs require a signature to replay", Session.EXIT_ERROR
-                )
+                raise ConfigError("Hangs require a signature to replay", Exit.ERROR)
             if "timeout" in ignore:
                 raise ConfigError(
-                    "Cannot ignore 'timeout' when detecting hangs", Session.EXIT_ERROR
+                    "Cannot ignore 'timeout' when detecting hangs", Exit.ERROR
                 )
         return is_hang
 
     @classmethod
-    def load_testcases(cls, path, load_prefs, subset=None):
+    def load_testcases(cls, path, subset=None):
         """Load TestCases.
 
         Args:
             path (str): Path to load.
-            load_prefs (bool): Load prefs.js file if available.
             subset (list(int)): Indices of tests to load when loading multiple
                                 tests.
         Returns:
-            list(TestCase): Loaded TestCases.
+            tuple (list(TestCase), AssetManager): Loaded TestCases and AssetManager.
         """
         LOG.debug("loading the TestCases")
-        testcases = TestCase.load(path, load_prefs)
+        testcases = TestCase.load(path)
         if not testcases:
             raise TestCaseLoadFailure("Failed to load TestCases")
+        # remove loaded assets and environment variables from test cases
+        assets = None
+        env_vars = None
+        for test in testcases:
+            if assets is None:
+                assets = test.pop_assets()
+            else:
+                test.pop_assets()
+            if not env_vars and test.env_vars:
+                env_vars = dict(test.env_vars)
+            test.env_vars.clear()
+        LOG.debug(
+            "loaded TestCase(s): %d, assets: %r, env vars: %r",
+            len(testcases),
+            assets is not None,
+            env_vars is not None,
+        )
         if subset:
             count = len(testcases)
             # deduplicate and limit requested indices to valid range
@@ -165,7 +177,7 @@ class ReplayManager:
             for test in testcases:
                 test.cleanup()
             testcases = selected
-        return testcases
+        return testcases, assets, env_vars
 
     @staticmethod
     def report_to_filesystem(path, results, tests=None):
@@ -173,7 +185,7 @@ class ReplayManager:
         known location.
 
         Args:
-            path (str): Location to write data.
+            path (Path): Location to write data.
             results (iterable): ReplayResult to output.
             tests (iterable): Testcases to output.
 
@@ -182,9 +194,7 @@ class ReplayManager:
         """
         others = list(x.report for x in results if not x.expected)
         if others:
-            reporter = FilesystemReporter(
-                pathjoin(path, "other_reports"), major_bucket=False
-            )
+            reporter = FilesystemReporter(path / "other_reports", major_bucket=False)
             for report in others:
                 reporter.submit(tests or [], report=report)
         expected = list(x for x in results if x.expected)
@@ -195,7 +205,7 @@ class ReplayManager:
                 for test, served in zip(tests, expected[0].served):
                     LOG.debug("calling test.purge_optional() with %r", served)
                     test.purge_optional(served)
-            reporter = FilesystemReporter(pathjoin(path, "reports"), major_bucket=False)
+            reporter = FilesystemReporter(path / "reports", major_bucket=False)
             for result in expected:
                 reporter.submit(tests or [], report=result.report)
 
@@ -209,6 +219,9 @@ class ReplayManager:
         expect_hang=False,
         idle_delay=0,
         idle_threshold=0,
+        launch_attempts=3,
+        on_iteration_cb=None,
+        post_launch_delay=None,
     ):
         """Run testcase replay.
 
@@ -226,12 +239,17 @@ class ReplayManager:
                                performed.
             idle_delay (int): Number of seconds to wait before polling for idle.
             idle_threshold (int): CPU usage threshold to mark the process as idle.
+            launch_attempts (int): Number of attempts to launch the browser.
+            on_iteration_cb (callable): Called every time a single iteration is run.
+            post_launch_delay (int): Time in seconds before test case is loaded after
+                                     the browser is launched.
 
         Returns:
             list: ReplayResults that were found running testcases.
         """
         assert idle_delay >= 0
         assert idle_threshold >= 0
+        assert launch_attempts > 0
         assert min_results > 0
         assert repeat > 0
         assert repeat >= min_results
@@ -240,29 +258,22 @@ class ReplayManager:
         assert self._harness is not None or len(testcases) == 1
         assert not expect_hang or self._signature is not None
 
-        if self.status is not None:
-            LOG.debug("clearing previous status data")
-            self.status.cleanup()
-        self.status = Status.start()
+        self.status = Status.start(db_file=self.STATUS_DB)
 
         server_map = ServerMap()
-        if self._harness is not None:
+        if self._harness is None:
+            server_map.set_redirect("grz_start", "grz_current_test", required=False)
+        else:
             server_map.set_dynamic_response(
-                "grz_harness", lambda: self._harness, mime_type="text/html"
+                "grz_harness", lambda _: self._harness, mime_type="text/html"
             )
+            server_map.set_redirect("grz_start", "grz_harness", required=False)
 
         # track unprocessed results
         reports = dict()
-        # track unpacked testcases
-        unpacked = list()
         try:
             sig_hash = Report.calc_hash(self._signature) if self._signature else None
             test_count = len(testcases)
-            LOG.debug("unpacking testcases (%d)...", test_count)
-            for test in testcases:
-                dst_path = mkdtemp(prefix="tc_", dir=grz_tmp("serve"))
-                unpacked.append(dst_path)
-                test.dump(dst_path)
             relaunch = min(self._relaunch, repeat)
             runner = Runner(
                 self.server,
@@ -274,25 +285,25 @@ class ReplayManager:
             # perform iterations
             for _ in range(repeat):
                 self.status.iteration += 1
+                if on_iteration_cb is not None:
+                    on_iteration_cb()
                 if self.target.closed:
                     if self._harness is None:
                         location = runner.location(
-                            "/grz_current_test", self.server.port
+                            "/grz_start",
+                            self.server.port,
+                            post_launch_delay=post_launch_delay,
                         )
                     else:
                         location = runner.location(
-                            "/grz_harness",
+                            "/grz_start",
                             self.server.port,
                             close_after=relaunch * test_count,
+                            post_launch_delay=post_launch_delay,
                             time_limit=time_limit,
                         )
-                    startup_error = False
-                    # The environment from the initial testcase is used because
-                    # a sequence of testcases is expected to be run without
-                    # relaunching the Target to match the functionality of
-                    # Grizzly. If this is not the case each TestCase should
-                    # be run individually.
-                    runner.launch(location, env_mod=testcases[0].env_vars)
+                    runner.launch(location, max_retries=launch_attempts)
+                    runner.post_launch(delay=post_launch_delay)
                 # run tests
                 durations = list()
                 served = list()
@@ -327,24 +338,25 @@ class ReplayManager:
                         self.ignore,
                         server_map,
                         testcases[test_idx],
-                        test_path=unpacked[test_idx],
                         wait_for_callback=self._harness is None,
                     )
                     durations.append(run_result.duration)
                     served.append(run_result.served)
-                    if run_result.status is not None or not run_result.attempted:
+                    if run_result.status != Result.NONE or not run_result.attempted:
                         break
                 if not run_result.attempted:
                     LOG.warning("Test case was not served")
-                    if run_result.initial:
-                        startup_error = True
-                        if run_result.status == RunResult.FAILED:
-                            # TODO: what is to best action to take in this case?
+                    if runner.initial:
+                        if run_result.status == Result.FOUND:
+                            # TODO: what is the best action to take in this case?
                             LOG.warning("Delayed startup failure detected")
                         else:
                             LOG.warning("Timeout too short? System too busy?")
                 # process run results
-                if run_result.status == RunResult.FAILED:
+                if run_result.status == Result.FOUND:
+                    # processing the result may take a few minutes (rr)
+                    # update console to show progress
+                    LOG.info("Processing result...")
                     # TODO: use self.target.create_report here
                     log_path = mkdtemp(prefix="logs_", dir=grz_tmp("logs"))
                     self.target.save_logs(log_path)
@@ -354,41 +366,40 @@ class ReplayManager:
                     # check signatures
                     if run_result.timeout:
                         short_sig = "Potential hang detected"
-                    else:
+                    elif report.crash_signature is not None:
                         short_sig = report.crash_info.createShortSignature()
+                    else:
+                        # FM crash signature creation failed
+                        short_sig = "Signature creation failed"
+
                     # set active signature
                     if (
-                        not startup_error
+                        not runner.startup_failure
                         and not self._any_crash
                         and not run_result.timeout
                         and self._signature is None
-                        and short_sig != "No crash detected"
                     ):
                         assert not expect_hang
                         LOG.debug("no signature given, using %r", short_sig)
                         self._signature = report.crash_signature
+
                     # bucket result
-                    if short_sig == "No crash detected":
-                        # TODO: verify report.major == "NO_STACK"
-                        # otherwise FM failed to parse the logs
-                        # TODO: change this to support hangs/timeouts, etc
-                        LOG.info("Result: No crash detected")
-                    elif not startup_error and (
+                    if not runner.startup_failure and (
                         self._any_crash
                         or self.check_match(self._signature, report, expect_hang)
                     ):
-                        self.status.count_result(short_sig)
+                        if sig_hash is not None:
+                            LOG.debug("using provided signature (hash) to bucket")
+                            bucket_hash = sig_hash
+                        else:
+                            bucket_hash = report.crash_hash
+                        self.status.results.count(bucket_hash, short_sig)
                         LOG.info(
                             "Result: %s (%s:%s)",
                             short_sig,
                             report.major[:8],
                             report.minor[:8],
                         )
-                        if sig_hash:
-                            LOG.debug("using provided signature (hash) to bucket")
-                            bucket_hash = sig_hash
-                        else:
-                            bucket_hash = report.crash_hash
                         if bucket_hash not in reports:
                             reports[bucket_hash] = ReplayResult(
                                 report, served, durations, True
@@ -419,22 +430,21 @@ class ReplayManager:
                     if report is not None:
                         report.cleanup()
                         report = None
-                elif run_result.status == RunResult.IGNORED:
+                elif run_result.status == Result.IGNORED:
                     self.status.ignored += 1
                     LOG.info("Result: Ignored (%d)", self.status.ignored)
 
-                if exit_early:
-                    # failed to meet minimum number of results
+                if exit_early and self.status.iteration < repeat:
+                    # check if failed to meet minimum number of results
                     if (
-                        repeat - self.status.iteration + self.status.results
+                        repeat - self.status.iteration + self.status.results.total
                         < min_results
                     ):
-                        if self.status.iteration < repeat:
-                            LOG.debug("skipping remaining attempts")
                         # failed to reproduce issue
+                        LOG.debug("skipping remaining attempts")
                         LOG.debug(
                             "results (%d) < minimum (%d), after %d attempts",
-                            self.status.results,
+                            self.status.results.total,
                             min_results,
                             self.status.iteration,
                         )
@@ -443,8 +453,8 @@ class ReplayManager:
                         # is an issue for now use relaunch=1
                         break
                     # check if complete (minimum number of results found)
-                    if self.status.results >= min_results:
-                        assert self.status.results == min_results
+                    if self.status.results.total >= min_results:
+                        assert self.status.results.total == min_results
                         assert (
                             sum(x.count for x in reports.values() if x.expected)
                             >= min_results
@@ -459,22 +469,28 @@ class ReplayManager:
                 # TODO: should we warn about large browser logs?
 
             # process results
-            results = list()
             if self._any_crash:
-                assert all(x.expected for x in reports.values())
-                if sum(x.count for x in reports.values()) >= min_results:
+                # add all results if min_results was reached
+                if sum(x.count for x in reports.values() if x.expected) >= min_results:
                     results = list(reports.values())
                 else:
+                    # add only unexpected results since min_results was not reached
+                    results = list()
+                    for report in reports.values():
+                        if report.expected:
+                            report.report.cleanup()
+                        else:
+                            results.append(report)
                     LOG.debug(
                         "%d (any_crash) less than minimum %d",
-                        self.status.results,
+                        self.status.results.total,
                         min_results,
                     )
-                    for report in reports.values():
-                        report.report.cleanup()
             else:
+                # there should be at most one expected bucket
                 assert sum(x.expected for x in reports.values()) <= 1
                 # filter out unreliable expected results
+                results = list()
                 for crash_hash, report in reports.items():
                     if report.expected and report.count < min_results:
                         LOG.debug(
@@ -501,9 +517,6 @@ class ReplayManager:
             return results
 
         finally:
-            # remove unpacked testcase data
-            for tc_path in unpacked:
-                rmtree(tc_path)
             # we don't want to clean up but we are not checking results
             self.target.close(force_close=True)
             # remove unprocessed reports
@@ -535,7 +548,7 @@ class ReplayManager:
             raise ConfigError(
                 "Timeout (%d) cannot be less than time limit (%d)"
                 % (timeout, time_limit),
-                Session.EXIT_ARGS,
+                Exit.ARGS,
             )
         return time_limit, timeout
 
@@ -548,10 +561,10 @@ class ReplayManager:
 
         LOG.info("Starting Grizzly Replay")
 
+        if args.headless:
+            LOG.info("Running browser headless (%s)", args.headless)
         if args.ignore:
             LOG.info("Ignoring: %s", ", ".join(args.ignore))
-        if args.xvfb:
-            LOG.info("Running with Xvfb")
         if args.pernosco:
             LOG.info("Running with RR (Pernosco mode)")
         elif args.rr:
@@ -565,23 +578,22 @@ class ReplayManager:
             signature = None
 
         try:
-            testcases = cls.load_testcases(
-                args.input, args.prefs is None, subset=args.test_index
+            testcases, assets, env_vars = cls.load_testcases(
+                str(args.input), subset=args.test_index
             )
         except TestCaseLoadFailure as exc:
             LOG.error("Error: %s", str(exc))
-            return Session.EXIT_ERROR
+            return Exit.ERROR
 
         results = None
         target = None
-        tmp_prefs = None
         try:
             if args.no_harness and len(testcases) > 1:
                 LOG.error(
                     "'--no-harness' cannot be used with multiple testcases. "
                     "Perhaps '--test-index' can help."
                 )
-                return Session.EXIT_ARGS
+                return Exit.ARGS
             # check if hangs are expected
             expect_hang = cls.expect_hang(args.ignore, signature, testcases)
             # check test time limit and timeout
@@ -604,30 +616,25 @@ class ReplayManager:
             LOG.debug("initializing the Target")
             target = load_plugin(args.platform, "grizzly_targets", Target)(
                 args.binary,
-                args.extension,
                 args.launch_timeout,
                 args.log_limit,
                 args.memory,
+                assets=assets,
+                headless=args.headless,
                 pernosco=args.pernosco,
                 rr=args.rr,
                 valgrind=args.valgrind,
-                xvfb=args.xvfb,
             )
-            # prioritize specified prefs.js file over included file
-            if args.prefs is not None:
-                for testcase in testcases:
-                    testcase.add_meta(TestFile.from_file(args.prefs, "prefs.js"))
-                LOG.info("Using specified prefs.js")
-                target.prefs = args.prefs
-            else:
-                for testcase in testcases:
-                    prefs_tf = testcase.get_file("prefs.js")
-                    if prefs_tf:
-                        tmp_prefs = mkdtemp(prefix="prefs_", dir=grz_tmp("replay"))
-                        prefs_tf.dump(tmp_prefs)
-                        LOG.info("Using prefs.js from testcase")
-                        target.prefs = pathjoin(tmp_prefs, "prefs.js")
-                        break
+            # local environ takes priority over environ loaded from test case
+            if env_vars is not None:
+                env_vars.update(target.environ)
+                target.environ = env_vars
+                env_vars = None
+            # TODO: support overriding existing assets
+            # prioritize specified assets over included
+            target.assets.add_batch(args.asset)
+            target.process_assets()
+
             LOG.debug("starting sapphire server")
             # launch HTTP server used to serve test cases
             with Sapphire(auto_close=1, timeout=timeout) as server:
@@ -647,7 +654,9 @@ class ReplayManager:
                         expect_hang=expect_hang,
                         idle_delay=args.idle_delay,
                         idle_threshold=args.idle_threshold,
+                        launch_attempts=args.launch_attempts,
                         min_results=args.min_crashes,
+                        post_launch_delay=args.post_launch_delay,
                         repeat=repeat,
                     )
             # handle results
@@ -657,18 +666,26 @@ class ReplayManager:
             else:
                 LOG.info("Failed to reproduce results")
             if args.logs and results:
+                # add target assets to test cases
+                if not target.assets.is_empty():
+                    for test in testcases:
+                        test.assets = target.assets
+                # add target environment variables
+                if target.filtered_environ():
+                    for test in testcases:
+                        test.env_vars = target.filtered_environ()
                 cls.report_to_filesystem(
                     args.logs, results, testcases if args.include_test else None
                 )
             # TODO: add fuzzmanager reporting
-            return Session.EXIT_SUCCESS if success else Session.EXIT_FAILURE
+            return Exit.SUCCESS if success else Exit.FAILURE
 
         except ConfigError as exc:
             LOG.error(str(exc))
             return exc.exit_code
 
         except KeyboardInterrupt:
-            return Session.EXIT_ABORT
+            return Exit.ABORT
 
         except (TargetLaunchError, TargetLaunchTimeout) as exc:
             LOG.error(str(exc))
@@ -677,7 +694,7 @@ class ReplayManager:
                 LOG.error("Logs can be found here %r", path)
                 reporter = FilesystemReporter(path, major_bucket=False)
                 reporter.submit([], exc.report)
-            return Session.EXIT_LAUNCH_FAILURE
+            return Exit.LAUNCH_FAILURE
 
         finally:
             LOG.info("Shutting down...")
@@ -689,6 +706,6 @@ class ReplayManager:
                 target.cleanup()
             for testcase in testcases:
                 testcase.cleanup()
-            if tmp_prefs is not None:
-                rmtree(tmp_prefs, ignore_errors=True)
+            if assets:
+                assets.cleanup()
             LOG.info("Done.")
