@@ -2,22 +2,30 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-# pylint: disable=protected-access
 """
 unit tests for grizzly.replay.main
 """
-from shutil import rmtree
-
 from pytest import mark
 
-from sapphire import SERVED_ALL
+from sapphire import Served
 
-from ..common import Report, TestCase, TestCaseLoadFailure
-from ..session import Session
-from ..target import Target, TargetLaunchError, TargetLaunchTimeout
+from ..common.reporter import Report
+from ..common.storage import TestCase, TestCaseLoadFailure
+from ..common.utils import Exit
+from ..target import (
+    AssetManager,
+    Result,
+    Target,
+    TargetLaunchError,
+    TargetLaunchTimeout,
+)
 from .replay import ReplayManager
 from .test_replay import _fake_save_logs
 
+pytestmark = mark.usefixtures(
+    "patch_collector", "tmp_path_grz_tmp", "tmp_path_replay_status_db"
+)
+
 
 def test_main_01(mocker, tmp_path):
     """test ReplayManager.main()"""
@@ -29,39 +37,39 @@ def test_main_01(mocker, tmp_path):
     serve_path = mocker.patch(
         "grizzly.replay.replay.Sapphire.serve_path",
         autospec=True,
-        return_value=(SERVED_ALL, ["test.html"]),  # passed to Target.detect_failure
+        return_value=(Served.ALL, ["test.html"]),  # passed to Target.check_result
     )
     # setup Target
-    load_target = mocker.patch("grizzly.replay.replay.load_plugin")
-    target = mocker.Mock(spec=Target, binary="bin", launch_timeout=30)
-    target.RESULT_FAILURE = Target.RESULT_FAILURE
-    target.RESULT_IGNORED = Target.RESULT_IGNORED
-    target.RESULT_NONE = Target.RESULT_NONE
-    target.detect_failure.side_effect = (
-        Target.RESULT_FAILURE,
-        Target.RESULT_NONE,
-        Target.RESULT_FAILURE,
-    )
+    load_target = mocker.patch("grizzly.replay.replay.load_plugin", autospec=True)
+    target = mocker.Mock(spec_set=Target, binary="bin", environ={}, launch_timeout=30)
+    target.assets = mocker.Mock(spec_set=AssetManager)
+    target.check_result.side_effect = (Result.FOUND, Result.NONE, Result.FOUND)
+    target.filtered_environ.return_value = {"ENV": "123"}
     target.save_logs = _fake_save_logs
     load_target.return_value.return_value = target
+    with TestCase("test.html", None, "adpt") as src:
+        src.env_vars["TEST_VAR"] = "100"
+        src.add_from_bytes(b"test", "test.html")
+        src.dump(str(tmp_path / "testcase"), include_details=True)
     # setup args
     log_path = tmp_path / "logs"
-    (tmp_path / "test.html").touch()
-    (tmp_path / "prefs.js").touch()
     (tmp_path / "sig.json").write_bytes(
         b'{"symptoms": [{"type": "crashAddress", "address": "0"}]}'
     )
     args = mocker.Mock(
+        any_crash=False,
+        asset=list(),
         fuzzmanager=False,
         idle_delay=0,
         idle_threshold=0,
         ignore=["fake", "timeout"],
-        input=str(tmp_path / "test.html"),
-        logs=str(log_path),
+        input=str(tmp_path / "testcase"),
+        launch_attempts=3,
+        logs=log_path,
         min_crashes=2,
         no_harness=False,
         pernosco=False,
-        prefs=str(tmp_path / "prefs.js"),
+        post_launch_delay=None,
         relaunch=1,
         repeat=4,
         rr=False,
@@ -71,14 +79,18 @@ def test_main_01(mocker, tmp_path):
         timeout=None,
         valgrind=False,
     )
-    assert ReplayManager.main(args) == Session.EXIT_SUCCESS
+    assert ReplayManager.main(args) == Exit.SUCCESS
     assert target.reverse.call_count == 1
     assert target.launch.call_count == 3
-    assert target.detect_failure.call_count == 3
+    assert target.check_result.call_count == 3
+    assert "TEST_VAR" in target.environ
     assert serve_path.call_count == 3
     assert load_target.call_count == 1
     assert target.close.call_count == 4
+    assert target.filtered_environ.call_count == 2
     assert target.cleanup.call_count == 1
+    assert target.assets.add.call_count == 0
+    assert target.assets.is_empty.call_count == 1
     assert log_path.is_dir()
     assert any(log_path.glob("reports/*/log_asan_blah.txt"))
     assert any(log_path.glob("reports/*/log_stderr.txt"))
@@ -92,26 +104,27 @@ def test_main_02(mocker, tmp_path):
     mocker.patch(
         "grizzly.replay.replay.Sapphire.serve_path",
         autospec=True,
-        return_value=(SERVED_ALL, ["test.html"]),  # passed to Target.detect_failure
+        return_value=(Served.ALL, ["test.html"]),  # passed to Target.check_result
     )
     # setup Target
     load_target = mocker.patch("grizzly.replay.replay.load_plugin")
-    target = mocker.Mock(spec=Target, binary="bin", launch_timeout=30)
-    target.RESULT_NONE = Target.RESULT_NONE
-    target.detect_failure.return_value = Target.RESULT_NONE
+    target = mocker.Mock(spec_set=Target, binary="bin", environ={}, launch_timeout=30)
+    target.check_result.return_value = Result.NONE
     load_target.return_value.return_value = target
     # setup args
     (tmp_path / "test.html").touch()
     args = mocker.Mock(
+        asset=list(),
         fuzzmanager=False,
         idle_delay=0,
         idle_threshold=0,
         ignore=["fake", "timeout"],
         input=str(tmp_path / "test.html"),
+        launch_attempts=3,
         min_crashes=2,
         no_harness=True,
         pernosco=False,
-        prefs=None,
+        post_launch_delay=None,
         relaunch=1,
         repeat=1,
         rr=False,
@@ -121,8 +134,9 @@ def test_main_02(mocker, tmp_path):
         timeout=None,
         valgrind=False,
     )
-    assert ReplayManager.main(args) == Session.EXIT_FAILURE
-    assert target.detect_failure.call_count == 1
+    assert ReplayManager.main(args) == Exit.FAILURE
+    assert target.check_result.call_count == 1
+    assert not target.environ
     assert target.close.call_count == 2
     assert target.cleanup.call_count == 1
 
@@ -136,12 +150,13 @@ def test_main_03(mocker):
     fake_tc = mocker.patch("grizzly.replay.replay.TestCase", autospec=True)
     # setup args
     args = mocker.Mock(
+        asset=list(),
         ignore=list(),
         input="test",
         min_crashes=1,
         no_harness=True,
         pernosco=False,
-        prefs=None,
+        post_launch_delay=None,
         relaunch=1,
         repeat=1,
         rr=False,
@@ -153,33 +168,35 @@ def test_main_03(mocker):
     )
     # user abort
     fake_load_target.side_effect = KeyboardInterrupt
-    assert ReplayManager.main(args) == Session.EXIT_ABORT
+    assert ReplayManager.main(args) == Exit.ABORT
     fake_load_target.reset_mock()
     # invalid test case
     fake_tc.load.side_effect = TestCaseLoadFailure
-    assert ReplayManager.main(args) == Session.EXIT_ERROR
+    assert ReplayManager.main(args) == Exit.ERROR
     assert fake_load_target.call_count == 0
     # no test cases
     fake_tc.load.side_effect = None
     fake_tc.load.return_value = list()
-    assert ReplayManager.main(args) == Session.EXIT_ERROR
+    assert ReplayManager.main(args) == Exit.ERROR
     assert fake_load_target.call_count == 0
     fake_load_target.reset_mock()
     # multiple test cases with --no-harness
-    fake_tc.load.return_value = [mocker.Mock(hang=False), mocker.Mock(hang=False)]
-    assert ReplayManager.main(args) == Session.EXIT_ARGS
+    fake_tc.load.return_value = [
+        mocker.Mock(spec_set=TestCase, env_vars={}, hang=False) for _ in range(2)
+    ]
+    assert ReplayManager.main(args) == Exit.ARGS
     assert fake_load_target.call_count == 0
     fake_load_target.reset_mock()
     # signature required replaying hang
-    fake_tc.load.return_value = [mocker.Mock(hang=True)]
-    assert ReplayManager.main(args) == Session.EXIT_ERROR
+    fake_tc.load.return_value = [mocker.Mock(spec_set=TestCase, env_vars={}, hang=True)]
+    assert ReplayManager.main(args) == Exit.ERROR
     assert fake_load_target.call_count == 0
     fake_load_target.reset_mock()
     # can't ignore timeout replaying hang
     args.ignore = ["timeout"]
     args.sig = "sig"
-    fake_tc.load.return_value = [mocker.Mock(hang=True)]
-    assert ReplayManager.main(args) == Session.EXIT_ERROR
+    fake_tc.load.return_value = [mocker.Mock(spec_set=TestCase, env_vars={}, hang=True)]
+    assert ReplayManager.main(args) == Exit.ERROR
     assert fake_sig.fromFile.call_count == 1
     assert fake_load_target.call_count == 0
     fake_load_target.reset_mock()
@@ -190,9 +207,12 @@ def test_main_04(mocker, tmp_path):
     mocker.patch("grizzly.replay.replay.FuzzManagerReporter", autospec=True)
     mocker.patch("grizzly.replay.replay.Sapphire", autospec=True)
     mocker.patch("grizzly.replay.replay.TestCase", autospec=True)
-    target = mocker.Mock(spec=Target, launch_timeout=30)
-    load_target = mocker.patch("grizzly.replay.replay.load_plugin", autospec=True)
-    load_target.return_value.return_value = target
+    target = mocker.NonCallableMock(spec_set=Target, launch_timeout=30)
+    mocker.patch(
+        "grizzly.replay.replay.load_plugin",
+        autospec=True,
+        return_value=mocker.Mock(spec_set=Target, return_value=target),
+    )
     fake_tmp = tmp_path / "grz_tmp"
     fake_tmp.mkdir()
     mocker.patch(
@@ -200,12 +220,13 @@ def test_main_04(mocker, tmp_path):
     )
     # setup args
     args = mocker.Mock(
+        asset=list(),
         ignore=list(),
         input="test",
         min_crashes=1,
         no_harness=True,
+        post_launch_delay=None,
         pernosco=False,
-        prefs=None,
         relaunch=1,
         repeat=1,
         rr=False,
@@ -218,45 +239,53 @@ def test_main_04(mocker, tmp_path):
     # target launch error
     fake_logs = tmp_path / "fake_report"
     fake_logs.mkdir()
-    report = mocker.Mock(spec=Report, prefix="fake_report", path=str(fake_logs))
+    report = mocker.Mock(spec_set=Report, prefix="fake_report", path=str(fake_logs))
     mocker.patch(
         "grizzly.replay.replay.ReplayManager.run",
         side_effect=TargetLaunchError("", report),
     )
-    assert ReplayManager.main(args) == Session.EXIT_LAUNCH_FAILURE
+    assert ReplayManager.main(args) == Exit.LAUNCH_FAILURE
     assert not fake_logs.is_dir()
     assert "fake_report_logs" in (x.name for x in fake_tmp.iterdir())
     # target launch timeout
     mocker.patch(
         "grizzly.replay.replay.ReplayManager.run", side_effect=TargetLaunchTimeout
     )
-    assert ReplayManager.main(args) == Session.EXIT_LAUNCH_FAILURE
+    assert ReplayManager.main(args) == Exit.LAUNCH_FAILURE
 
 
 def test_main_05(mocker, tmp_path):
-    """test ReplayManager.main() loading/generating prefs.js"""
+    """test ReplayManager.main() loading specified assets"""
     serve_path = mocker.patch(
         "grizzly.replay.replay.Sapphire.serve_path",
         autospec=True,
-        return_value=(None, ["test.html"]),  # passed to Target.detect_failure
+        return_value=(None, ["test.html"]),  # passed to Target.check_result
     )
     # setup Target
-    target = mocker.Mock(spec=Target, binary="bin", launch_timeout=30)
-    target.RESULT_FAILURE = Target.RESULT_FAILURE
-    target.detect_failure.return_value = Target.RESULT_FAILURE
+    target = mocker.NonCallableMock(spec_set=Target, binary="bin", launch_timeout=30)
+    target.check_result.return_value = Result.FOUND
+    target.filtered_environ.return_value = dict()
     target.monitor.is_healthy.return_value = False
     target.save_logs = _fake_save_logs
-    load_target = mocker.patch("grizzly.replay.replay.load_plugin")
-    load_target.return_value.return_value = target
+    mocker.patch(
+        "grizzly.replay.replay.load_plugin",
+        autospec=True,
+        return_value=mocker.Mock(spec_set=Target, return_value=target),
+    )
+    asset = tmp_path / "sample_asset"
+    asset.touch()
     # setup args
     args = mocker.Mock(
+        asset=[["from_cmdline", str(asset)]],
         fuzzmanager=False,
         idle_delay=0,
         idle_threshold=0,
         ignore=list(),
+        launch_attempts=3,
         min_crashes=1,
         no_harness=True,
         pernosco=False,
+        post_launch_delay=None,
         relaunch=1,
         repeat=1,
         rr=False,
@@ -267,7 +296,7 @@ def test_main_05(mocker, tmp_path):
         valgrind=False,
     )
     log_path = tmp_path / "logs"
-    args.logs = str(log_path)
+    args.logs = log_path
     input_path = tmp_path / "input"
     input_path.mkdir()
     # build a test case
@@ -277,61 +306,33 @@ def test_main_05(mocker, tmp_path):
         src.add_from_file(str(entry_point))
         src.dump(str(input_path), include_details=True)
     args.input = str(input_path)
-
-    # test no specified prefs.js
-    args.prefs = None
-    assert ReplayManager.main(args) == Session.EXIT_SUCCESS
+    with AssetManager(base_path=str(tmp_path)) as assets:
+        target.assets = assets
+        assert ReplayManager.main(args) == Exit.SUCCESS
+        assert "from_cmdline" in target.assets.assets
     assert target.launch.call_count == 1
-    assert target.detect_failure.call_count == 1
+    assert target.check_result.call_count == 1
+    assert target.filtered_environ.call_count == 1
     assert serve_path.call_count == 1
     assert log_path.is_dir()
-    assert not any(log_path.glob("**/prefs.js"))
-
-    target.reset_mock()
-    serve_path.reset_mock()
-    rmtree(str(log_path), ignore_errors=True)
-
-    # test included prefs.js
-    (input_path / "prefs.js").write_bytes(b"included")
-    assert ReplayManager.main(args) == Session.EXIT_SUCCESS
-    assert target.launch.call_count == 1
-    assert target.detect_failure.call_count == 1
-    assert serve_path.call_count == 1
-    assert log_path.is_dir()
-    prefs = next(log_path.glob("**/prefs.js"))
-    assert prefs.read_bytes() == b"included"
-
-    target.reset_mock()
-    serve_path.reset_mock()
-    rmtree(str(log_path), ignore_errors=True)
-
-    # test specified prefs.js
-    (tmp_path / "prefs.js").write_bytes(b"specified")
-    args.prefs = str(tmp_path / "prefs.js")
-    assert ReplayManager.main(args) == Session.EXIT_SUCCESS
-    assert target.launch.call_count == 1
-    assert target.detect_failure.call_count == 1
-    assert serve_path.call_count == 1
-    assert log_path.is_dir()
-    prefs = next(log_path.glob("**/prefs.js"))
-    assert prefs.read_bytes() == b"specified"
+    assert any(log_path.glob("**/sample_asset"))
 
 
 @mark.parametrize(
     "arg_timelimit, arg_timeout, test_timelimit, result",
     [
         # use default test time limit and timeout values (test missing time limit)
-        (None, None, None, Session.EXIT_FAILURE),
+        (None, None, None, Exit.FAILURE),
         # use min test time limit and default timeout values
-        (None, None, 1, Session.EXIT_FAILURE),
+        (None, None, 1, Exit.FAILURE),
         # set test time limit
-        (10, None, None, Session.EXIT_FAILURE),
+        (10, None, None, Exit.FAILURE),
         # set both test time limit and timeout to the same value
-        (10, 10, None, Session.EXIT_FAILURE),
+        (10, 10, None, Exit.FAILURE),
         # set timeout greater than test time limit
-        (10, 11, None, Session.EXIT_FAILURE),
+        (10, 11, None, Exit.FAILURE),
         # set test time limit greater than timeout
-        (11, 10, None, Session.EXIT_ARGS),
+        (11, 10, None, Exit.ARGS),
     ],
 )
 def test_main_06(mocker, tmp_path, arg_timelimit, arg_timeout, test_timelimit, result):
@@ -341,14 +342,16 @@ def test_main_06(mocker, tmp_path, arg_timelimit, arg_timeout, test_timelimit, r
     mocker.patch(
         "grizzly.replay.replay.Sapphire.serve_path",
         autospec=True,
-        return_value=(SERVED_ALL, ["test.html"]),  # passed to Target.detect_failure
+        return_value=(Served.ALL, ["test.html"]),  # passed to Target.check_result
     )
     # setup Target
-    target = mocker.Mock(spec=Target, binary="bin", launch_timeout=30)
-    target.RESULT_NONE = Target.RESULT_NONE
-    target.detect_failure.return_value = Target.RESULT_NONE
-    load_target = mocker.patch("grizzly.replay.replay.load_plugin")
-    load_target.return_value.return_value = target
+    target = mocker.NonCallableMock(spec_set=Target, binary="bin", launch_timeout=30)
+    target.check_result.return_value = Result.NONE
+    mocker.patch(
+        "grizzly.replay.replay.load_plugin",
+        autospec=True,
+        return_value=mocker.Mock(spec_set=Target, return_value=target),
+    )
     # create test to load
     with TestCase("test.html", None, None) as test:
         test_file = tmp_path / "test.html"
@@ -360,15 +363,17 @@ def test_main_06(mocker, tmp_path, arg_timelimit, arg_timeout, test_timelimit, r
         test.dump(str(replay_path), include_details=True)
     # setup args
     args = mocker.Mock(
+        asset=list(),
         fuzzmanager=False,
         idle_delay=0,
         idle_threshold=0,
         ignore=["timeout"],
         input=str(replay_path),
+        launch_attempts=3,
         min_crashes=2,
         no_harness=True,
         pernosco=False,
-        prefs=None,
+        post_launch_delay=None,
         relaunch=1,
         repeat=1,
         rr=False,
@@ -403,26 +408,30 @@ def test_main_07(mocker, tmp_path, pernosco, rr, valgrind, no_harness):
     mocker.patch(
         "grizzly.replay.replay.Sapphire.serve_path",
         autospec=True,
-        return_value=(SERVED_ALL, ["test.html"]),  # passed to Target.detect_failure
+        return_value=(Served.ALL, ["test.html"]),  # passed to Target.check_result
     )
     # setup Target
-    load_target = mocker.patch("grizzly.replay.replay.load_plugin")
-    target = mocker.Mock(spec=Target, binary="bin", launch_timeout=30)
-    target.RESULT_NONE = Target.RESULT_NONE
-    target.detect_failure.return_value = Target.RESULT_NONE
-    load_target.return_value.return_value = target
+    target = mocker.NonCallableMock(spec_set=Target, binary="bin", launch_timeout=30)
+    target.check_result.return_value = Result.NONE
+    load_target = mocker.patch(
+        "grizzly.replay.replay.load_plugin",
+        autospec=True,
+        return_value=mocker.Mock(spec_set=Target, return_value=target),
+    )
     # setup args
     (tmp_path / "test.html").touch()
     args = mocker.Mock(
+        asset=list(),
         fuzzmanager=False,
         idle_delay=0,
         idle_threshold=0,
         ignore=["fake", "timeout"],
         input=str(tmp_path / "test.html"),
+        launch_attempts=3,
         min_crashes=2,
         no_harness=no_harness,
         pernosco=pernosco,
-        prefs=None,
+        post_launch_delay=None,
         relaunch=1,
         repeat=1,
         rr=rr,
@@ -434,8 +443,8 @@ def test_main_07(mocker, tmp_path, pernosco, rr, valgrind, no_harness):
     )
     # maximum one debugger allowed at a time
     assert sum((pernosco, rr, valgrind)) < 2, "test broken!"
-    assert ReplayManager.main(args) == Session.EXIT_FAILURE
-    assert target.detect_failure.call_count == 1
+    assert ReplayManager.main(args) == Exit.FAILURE
+    assert target.check_result.call_count == 1
     assert target.close.call_count == 2
     assert target.cleanup.call_count == 1
     assert load_target.return_value.call_args[-1]["pernosco"] == pernosco
