@@ -2,18 +2,32 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from abc import ABCMeta, abstractmethod, abstractproperty
+from abc import ABCMeta, abstractmethod
+from enum import Enum, unique
 from logging import getLogger
+from os import environ
 from os.path import isfile
 from threading import Lock
 
-__all__ = ("Target", "TargetError", "TargetLaunchError")
+from ..common.utils import grz_tmp
+from .assets import AssetManager
+
+__all__ = ("Result", "Target", "TargetError", "TargetLaunchError")
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith", "Jesse Schwartzentruber"]
 
 LOG = getLogger(__name__)
 
 
+@unique
+class Result(Enum):
+    """Target results codes"""
+
+    NONE = 0
+    FOUND = 1
+    IGNORED = 2
+
+
 class TargetError(Exception):
     """Raised by Target"""
 
@@ -31,31 +45,32 @@ class TargetLaunchTimeout(TargetError):
 
 
 class Target(metaclass=ABCMeta):
-    RESULT_NONE = 0
-    RESULT_FAILURE = 1
-    RESULT_IGNORED = 2
+    SUPPORTED_ASSETS = None
+    TRACKED_ENVVARS = ()
 
     __slots__ = (
+        "_assets",
         "_lock",
         "_monitor",
-        "_prefs",
         "binary",
-        "extension",
+        "environ",
         "launch_timeout",
         "log_limit",
         "memory_limit",
     )
 
-    def __init__(self, binary, extension, launch_timeout, log_limit, memory_limit):
+    def __init__(self, binary, launch_timeout, log_limit, memory_limit, assets=None):
+        assert launch_timeout > 0
         assert log_limit >= 0
         assert memory_limit >= 0
         assert binary is not None and isfile(binary)
+        assert assets is None or isinstance(assets, AssetManager)
+        self._assets = assets if assets else AssetManager(base_path=grz_tmp("target"))
         self._lock = Lock()
         self._monitor = None
-        self._prefs = None
         self.binary = binary
-        self.extension = extension
-        self.launch_timeout = max(launch_timeout, 300)
+        self.environ = self.scan_environment(dict(environ), self.TRACKED_ENVVARS)
+        self.launch_timeout = launch_timeout
         self.log_limit = log_limit
         self.memory_limit = memory_limit
 
@@ -65,18 +80,35 @@ class Target(metaclass=ABCMeta):
     def __exit__(self, *exc):
         self.cleanup()
 
-    def add_abort_token(self, _token):  # pylint: disable=no-self-use
-        LOG.warning("add_abort_token() not implemented!")
+    @property
+    def assets(self):
+        return self._assets
+
+    @assets.setter
+    def assets(self, assets):
+        self._assets.cleanup()
+        assert isinstance(assets, AssetManager)
+        self._assets = assets
 
     @abstractmethod
-    def cleanup(self):
+    def _cleanup(self):
         pass
 
+    @abstractmethod
+    def check_result(self, ignored):
+        pass
+
+    def cleanup(self):
+        # call target specific _cleanup first
+        self._cleanup()
+        self._assets.cleanup()
+
     @abstractmethod
     def close(self, force_close=False):
         pass
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def closed(self):
         pass
 
@@ -84,13 +116,13 @@ class Target(metaclass=ABCMeta):
     def create_report(self, is_hang=False):
         pass
 
-    @abstractmethod
-    def detect_failure(self, ignored):
-        pass
-
     def dump_coverage(self, _timeout=0):  # pylint: disable=no-self-use
         LOG.warning("dump_coverage() is not supported!")
 
+    def filtered_environ(self):
+        """Used to collect the environment to add to a testcase"""
+        return dict(self.environ)
+
     @abstractmethod
     def handle_hang(self, ignore_idle=True):
         pass
@@ -101,26 +133,36 @@ class Target(metaclass=ABCMeta):
         return False
 
     @abstractmethod
-    def launch(self, _location, _env_mod=None):
+    def launch(self, location):
         pass
 
     def log_size(self):  # pylint: disable=no-self-use
         LOG.debug("log_size() not implemented! returning 0")
         return 0
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def monitor(self):
         pass
 
-    # TODO: better meta file handling
-    @abstractproperty
-    def prefs(self):
+    @abstractmethod
+    def process_assets(self):
         pass
 
     def reverse(self, remote, local):
         # remote->device, local->desktop
         pass
 
+    @staticmethod
+    def scan_environment(to_scan, tracked):
+        # scan environment for tracked environment variables
+        env = dict()
+        if tracked:
+            for var in tracked:
+                if var in to_scan:
+                    env[var] = to_scan[var]
+        return env
+
     @abstractmethod
     def save_logs(self, *args, **kwargs):
         pass
