@@ -6,16 +6,19 @@
 """
 unit tests for grizzly.Session
 """
-from itertools import count
+from itertools import chain, count, repeat
 
 from pytest import mark, raises
 
-from sapphire import SERVED_ALL, SERVED_NONE, SERVED_TIMEOUT, Sapphire
+from sapphire import Sapphire, Served
 
 from .adapter import Adapter
-from .common import Report, Reporter, RunResult, Status
+from .common.reporter import Report, Reporter
+from .common.runner import RunResult
 from .session import LogOutputLimiter, Session, SessionError
-from .target import Target, TargetLaunchError
+from .target import AssetManager, Result, Target, TargetLaunchError
+
+pytestmark = mark.usefixtures("patch_collector", "tmp_path_status_db")
 
 
 class SimpleAdapter(Adapter):
@@ -32,7 +35,7 @@ class SimpleAdapter(Adapter):
     def generate(self, testcase, _server_map):
         assert testcase.adapter_name == self.name
         testcase.input_fname = self.fuzz["input"]
-        testcase.add_from_data("test", testcase.landing_page)
+        testcase.add_from_bytes(b"test", testcase.landing_page)
         if self.remaining is not None:
             assert self.remaining > 0
             self.remaining -= 1
@@ -61,16 +64,16 @@ class SimpleAdapter(Adapter):
         (True, False, False, 1, 0, 1),
     ],
 )
-def test_session_01(
-    mocker, tmp_path, harness, profiling, coverage, relaunch, iters, runtime
-):
+def test_session_01(mocker, harness, profiling, coverage, relaunch, iters, runtime):
     """test Session with typical fuzzer Adapter"""
-    Status.PATH = str(tmp_path)
     mocker.patch("grizzly.common.status.time", side_effect=count(start=1.0, step=1.0))
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    prefs = tmp_path / "prefs.js"
-    prefs.touch()
-    target = mocker.Mock(spec=Target, launch_timeout=30, prefs=str(prefs))
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+        launch_timeout=30,
+    )
     target.log_size.return_value = 1000
     target.monitor.launches = 1
     # avoid shutdown delay
@@ -92,7 +95,7 @@ def test_session_01(
         relaunch=relaunch,
     ) as session:
         server.serve_path = lambda *a, **kv: (
-            SERVED_ALL,
+            Served.ALL,
             [session.iomanager.page_name(offset=-1)],
         )
         session.run(
@@ -105,7 +108,7 @@ def test_session_01(
         assert session.status.iteration == max_iters
         assert session.status.test_name == "file.bin"
         assert target.close.call_count == max_iters / relaunch
-        assert target.detect_failure.call_count == max_iters
+        assert target.check_result.call_count == max_iters
         assert target.handle_hang.call_count == 0
         if profiling:
             assert any(session.status.profile_entries()) == profiling
@@ -132,11 +135,15 @@ def test_session_01(
         (True, 10, 10),
     ],
 )
-def test_session_02(tmp_path, mocker, harness, relaunch, remaining):
+def test_session_02(mocker, harness, relaunch, remaining):
     """test Session with playback Adapter"""
-    Status.PATH = str(tmp_path)
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    target = mocker.Mock(spec=Target, launch_timeout=30, prefs=None)
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+        launch_timeout=30,
+    )
     # calculate if the target is 'closed' based on relaunch
     type(target).closed = mocker.PropertyMock(
         side_effect=((x % relaunch == 0) for x in range(remaining))
@@ -153,44 +160,46 @@ def test_session_02(tmp_path, mocker, harness, relaunch, remaining):
         relaunch=relaunch,
     ) as session:
         server.serve_path = lambda *a, **kv: (
-            SERVED_ALL,
+            Served.ALL,
             [session.iomanager.page_name(offset=-1)],
         )
         session.run([], 10)
         assert session.status.iteration == remaining
         assert session.status.test_name is None
-        assert target.detect_failure.call_count == remaining
+        assert target.check_result.call_count == remaining
         assert target.handle_hang.call_count == 0
 
 
 @mark.parametrize(
-    "harness, report_size, relaunch, iters",
+    "harness, report_size, relaunch, iters, has_sig",
     [
         # with harness, collect 1 test case
-        (True, 1, 1, 1),
+        (True, 1, 1, 1, True),
         # with harness, collect 2 test cases
-        (True, 2, 2, 2),
+        (True, 2, 2, 2, True),
         # with harness, collect 2 test cases,
-        (True, 2, 3, 3),
+        (True, 2, 3, 3, True),
         # with harness, collect 3 test cases, relaunch 1
-        (True, 3, 1, 3),
+        (True, 3, 1, 3, True),
         # without harness, collect 1 test case
-        (False, 1, 1, 1),
+        (False, 1, 1, 1, True),
         # without harness, collect 1 test case, 3 iterations
-        (False, 1, 1, 3),
+        (False, 1, 1, 3, True),
+        # with harness, collect 1 test case, failed to generate FM signature
+        (True, 1, 1, 1, False),
     ],
 )
-def test_session_03(mocker, tmp_path, harness, report_size, relaunch, iters):
+def test_session_03(mocker, tmp_path, harness, report_size, relaunch, iters, has_sig):
     """test Session - detecting failure"""
-    Status.PATH = str(tmp_path)
     adapter = SimpleAdapter(harness)
-    reporter = mocker.Mock(spec=Reporter)
-    report = mocker.Mock(spec=Report, major="major123", minor="minor456")
-    report.crash_info.createShortSignature.return_value = "[@ sig]"
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    target = mocker.Mock(spec=Target, launch_timeout=30, prefs=None)
-    target.RESULT_FAILURE = Target.RESULT_FAILURE
-    target.RESULT_NONE = Target.RESULT_NONE
+    reporter = mocker.Mock(spec_set=Reporter)
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+        launch_timeout=30,
+    )
     target.monitor.launches = 1
     # avoid shutdown delay
     target.monitor.is_healthy.return_value = False
@@ -199,47 +208,66 @@ def test_session_03(mocker, tmp_path, harness, report_size, relaunch, iters):
         side_effect=((x % relaunch == 0) for x in range(iters))
     )
     # failure is on final iteration
-    target.detect_failure.side_effect = [
-        Target.RESULT_NONE for x in range(iters - 1)
-    ] + [Target.RESULT_FAILURE]
+    target.check_result.side_effect = chain(
+        repeat(Result.NONE, iters - 1), (Result.FOUND,)
+    )
     target.log_size.return_value = 1
+    # create Report
+    log_path = tmp_path / "logs"
+    log_path.mkdir()
+    (log_path / "log_stderr.txt").write_bytes(b"STDERR log")
+    (log_path / "log_stdout.txt").write_bytes(b"STDOUT log")
+    if has_sig:
+        with (log_path / "log_asan_blah.txt").open("w") as log_fp:
+            log_fp.write("==1==ERROR: AddressSanitizer: ")
+            log_fp.write("SEGV on unknown address 0x0 (pc 0x0 bp 0x0 sp 0x0 T0)\n")
+            log_fp.write("    #0 0xbad000 in foo /file1.c:123:234\n")
+            log_fp.write("    #1 0x1337dd in bar /file2.c:1806:19\n")
+    report = Report(str(log_path), "bin")
     target.create_report.return_value = report
     with Session(
         adapter, reporter, server, target, relaunch=relaunch, report_size=report_size
     ) as session:
         server.serve_path = lambda *a, **kv: (
-            SERVED_ALL,
+            Served.ALL,
             [session.iomanager.page_name(offset=-1)],
         )
         session.run([], 10, input_path="file.bin", iteration_limit=iters)
         assert reporter.submit.call_count == 1
         assert len(reporter.submit.call_args[0][0]) == min(report_size, relaunch)
-        assert reporter.submit.call_args[0][1].major == "major123"
 
 
-def test_session_04(mocker, tmp_path):
+def test_session_04(mocker):
     """test Adapter creating invalid test case"""
 
     class FuzzAdapter(Adapter):
         def generate(self, _testcase, _server_map):
             pass
 
-    Status.PATH = str(tmp_path)
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    server.serve_path.return_value = (SERVED_NONE, [])
-    target = mocker.Mock(spec=Target, launch_timeout=30, prefs=None)
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    server.serve_path.return_value = (Served.NONE, [])
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+        launch_timeout=30,
+    )
     target.monitor.launches = 1
     with Session(FuzzAdapter("fuzz"), None, server, target) as session:
         with raises(SessionError, match="Test case is missing landing page"):
             session.run([], 10)
 
 
-def test_session_05(mocker, tmp_path):
+def test_session_05(mocker):
     """test Target not requesting landing page"""
-    Status.PATH = str(tmp_path)
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    server.serve_path.return_value = (SERVED_TIMEOUT, [])
-    target = mocker.Mock(spec=Target, launch_timeout=30, prefs=None)
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    server.serve_path.return_value = (Served.TIMEOUT, [])
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+        launch_timeout=30,
+    )
     target.monitor.launches = 1
     with Session(SimpleAdapter(False), None, server, target) as session:
         with raises(SessionError, match="Please check Adapter and Target"):
@@ -257,19 +285,23 @@ def test_session_05(mocker, tmp_path):
         (False, 1),
     ],
 )
-def test_session_06(mocker, tmp_path, harness, report_size):
+def test_session_06(mocker, harness, report_size):
     """test Session - handle Target delayed failures"""
-    Status.PATH = str(tmp_path)
-    reporter = mocker.Mock(spec=Reporter)
-    report = mocker.Mock(spec=Report, major="major123", minor="minor456")
+    reporter = mocker.Mock(spec_set=Reporter)
+    report = mocker.Mock(
+        spec_set=Report, major="major123", minor="minor456", crash_hash="1234"
+    )
     report.crash_info.createShortSignature.return_value = "[@ sig]"
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    target = mocker.Mock(spec=Target, launch_timeout=30, prefs=None)
-    target.RESULT_FAILURE = Target.RESULT_FAILURE
-    target.RESULT_NONE = Target.RESULT_NONE
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+        launch_timeout=30,
+    )
     target.monitor.launches = 1
     type(target).closed = mocker.PropertyMock(side_effect=(True, False))
-    target.detect_failure.side_effect = (Target.RESULT_NONE, Target.RESULT_FAILURE)
+    target.check_result.side_effect = (Result.NONE, Result.FOUND)
     target.log_size.return_value = 1
     target.create_report.return_value = report
     with Session(
@@ -281,8 +313,8 @@ def test_session_06(mocker, tmp_path, harness, report_size):
         report_size=report_size,
     ) as session:
         server.serve_path.side_effect = (
-            (SERVED_ALL, [session.iomanager.page_name()]),
-            (SERVED_NONE, []),
+            (Served.ALL, [session.iomanager.page_name()]),
+            (Served.NONE, []),
         )
         session.run([], 10, iteration_limit=2)
         assert reporter.submit.call_count == 1
@@ -294,48 +326,54 @@ def test_session_06(mocker, tmp_path, harness, report_size):
     "srv_results, target_result, ignored, results",
     [
         # delayed startup crash
-        (SERVED_NONE, Target.RESULT_FAILURE, 0, 1),
+        (Served.NONE, Result.FOUND, 0, 1),
         # startup hang/unresponsive
-        (SERVED_TIMEOUT, Target.RESULT_NONE, 1, 0),
+        (Served.TIMEOUT, Result.NONE, 1, 0),
     ],
 )
-def test_session_07(mocker, tmp_path, srv_results, target_result, ignored, results):
+def test_session_07(mocker, srv_results, target_result, ignored, results):
     """test Session.run() - initial test case was not served"""
-    Status.PATH = str(tmp_path)
-    report = mocker.Mock(spec=Report, major="major123", minor="minor456")
+    report = mocker.Mock(
+        spec_set=Report, major="major123", minor="minor456", crash_hash="123"
+    )
     report.crash_info.createShortSignature.return_value = "[@ sig]"
-    reporter = mocker.Mock(spec=Reporter)
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    target = mocker.Mock(spec=Target, closed=True, launch_timeout=30, prefs=None)
-    target.RESULT_FAILURE = Target.RESULT_FAILURE
-    target.RESULT_NONE = Target.RESULT_NONE
+    reporter = mocker.Mock(spec_set=Reporter)
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        closed=True,
+        environ=dict(),
+        launch_timeout=30,
+    )
     target.monitor.launches = 1
-    target.detect_failure.side_effect = (target_result,)
+    target.check_result.side_effect = (target_result,)
     target.create_report.return_value = report
     with Session(SimpleAdapter(False), reporter, server, target) as session:
         server.serve_path.return_value = (srv_results, [])
         with raises(SessionError, match="Please check Adapter and Target"):
             session.run([], 10, iteration_limit=2)
         assert session.status.iteration == 1
-        assert session.status.results == results
+        assert session.status.results.total == results
         assert session.status.ignored == ignored
         assert reporter.submit.call_count == results
-        assert target.detect_failure.call_count == results
+        assert target.check_result.call_count == results
         assert target.handle_hang.call_count == ignored
 
 
-def test_session_08(tmp_path, mocker):
+def test_session_08(mocker):
     """test Session.run() ignoring failures"""
-    Status.PATH = str(tmp_path)
-    result = RunResult([], 0.1, status=RunResult.IGNORED)
+    result = RunResult([], 0.1, status=Result.IGNORED)
     result.attempted = True
     runner = mocker.patch("grizzly.session.Runner", autospec=True)
     runner.return_value.run.return_value = result
-    mocker.patch("grizzly.session.TestFile", autospec=True)
-    adapter = mocker.Mock(spec=Adapter, remaining=None)
+    runner.return_value.startup_failure = False
+    adapter = mocker.Mock(spec_set=Adapter, remaining=None)
     adapter.IGNORE_UNSERVED = False
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    target = mocker.Mock(spec=Target, prefs=None)
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target, assets=mocker.Mock(spec_set=AssetManager), environ=dict()
+    )
     target.monitor.launches = 1
     with Session(adapter, None, server, target) as session:
         session.run([], 10, iteration_limit=1)
@@ -344,42 +382,47 @@ def test_session_08(tmp_path, mocker):
         assert adapter.on_timeout.call_count == 0
         assert target.create_report.call_count == 0
         assert session.status.iteration == 1
-        assert session.status.results == 0
+        assert session.status.results.total == 0
         assert session.status.ignored == 1
 
 
-def test_session_09(tmp_path, mocker):
+def test_session_09(mocker):
     """test Session.run() handle TargetLaunchError"""
-    Status.PATH = str(tmp_path)
-    report = mocker.Mock(spec=Report, major="major123", minor="minor456")
+    report = mocker.Mock(
+        spec_set=Report, major="major123", minor="minor456", crash_hash="123"
+    )
+    report.crash_info.createShortSignature.return_value = "[@ sig]"
     runner = mocker.patch("grizzly.session.Runner", autospec=True)
     runner.return_value.launch.side_effect = TargetLaunchError("test", report)
-    mocker.patch("grizzly.session.TestFile", autospec=True)
-    adapter = mocker.Mock(spec=Adapter, remaining=None)
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    target = mocker.Mock(spec=Target)
+    runner.return_value.startup_failure = True
+    adapter = mocker.Mock(spec_set=Adapter, remaining=None)
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(spec_set=Target)
     target.monitor.launches = 1
-    with Session(adapter, mocker.Mock(spec=Reporter), server, target) as session:
+    with Session(adapter, mocker.Mock(spec_set=Reporter), server, target) as session:
         with raises(TargetLaunchError, match="test"):
             session.run([], 10, iteration_limit=2)
         assert session.status.iteration == 1
-        assert session.status.results == 1
+        assert session.status.results.total == 1
         assert session.status.ignored == 0
 
 
-def test_session_10(tmp_path, mocker):
+def test_session_10(mocker):
     """test Session.run() report hang"""
-    Status.PATH = str(tmp_path)
-    result = RunResult([], 60.0, status=RunResult.FAILED, timeout=True)
+    result = RunResult([], 60.0, status=Result.FOUND, timeout=True)
     result.attempted = True
     runner = mocker.patch("grizzly.session.Runner", autospec=True)
     runner.return_value.run.return_value = result
-    mocker.patch("grizzly.session.TestFile", autospec=True)
-    adapter = mocker.Mock(spec=Adapter, remaining=None)
-    report = mocker.Mock(spec=Report, major="major123", minor="minor456")
-    reporter = mocker.Mock(spec=Reporter)
-    server = mocker.Mock(spec=Sapphire, port=0x1337)
-    target = mocker.Mock(spec=Target, prefs=None)
+    runner.return_value.startup_failure = False
+    adapter = mocker.Mock(spec_set=Adapter, remaining=None)
+    report = mocker.Mock(spec_set=Report, major="major123", minor="minor456")
+    reporter = mocker.Mock(spec_set=Reporter)
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+    )
     target.monitor.launches = 1
     target.create_report.return_value = report
     with Session(adapter, reporter, server, target) as session:
@@ -390,10 +433,103 @@ def test_session_10(tmp_path, mocker):
         assert target.create_report.call_count == 1
         assert reporter.submit.call_count == 1
         assert session.status.iteration == 1
-        assert session.status.results == 1
+        assert session.status.results.total == 1
         assert session.status.ignored == 0
 
 
+@mark.parametrize(
+    "harness, report_size, relaunch, iters, report_limit",
+    [
+        # no limit, always submit reports
+        (True, 1, 1, 10, 0),
+        # limit, only submit initial reports
+        (True, 1, 1, 10, 5),
+    ],
+)
+def test_session_11(mocker, harness, report_size, relaunch, iters, report_limit):
+    """test Session - limit report submission"""
+    adapter = SimpleAdapter(harness)
+    reporter = mocker.Mock(spec_set=Reporter)
+    report = mocker.Mock(spec_set=Report, major="abc", minor="def", crash_hash="123")
+    report.crash_info.createShortSignature.return_value = "[@ sig]"
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+        launch_timeout=30,
+    )
+    target.monitor.launches = 1
+    # avoid shutdown delay
+    target.monitor.is_healthy.return_value = False
+    type(target).closed = mocker.PropertyMock(return_value=True)
+    target.check_result.return_value = Result.FOUND
+    target.log_size.return_value = 1
+    target.create_report.return_value = report
+    with Session(
+        adapter, reporter, server, target, report_limit=report_limit, relaunch=1
+    ) as session:
+        server.serve_path = lambda *a, **kv: (
+            Served.ALL,
+            [session.iomanager.page_name(offset=-1)],
+        )
+        session.run([], 10, input_path="file.bin", iteration_limit=iters)
+        if report_limit > 0:
+            assert reporter.submit.call_count == report_limit
+        else:
+            assert reporter.submit.call_count == iters
+        assert len(reporter.submit.call_args[0][0]) == min(report_size, relaunch)
+        assert reporter.submit.call_args[0][1].major == "abc"
+
+
+@mark.parametrize(
+    "harness, iters, result_limit, results",
+    [
+        # hit result limit
+        (True, 0, 1, 1),
+        # don't hit result limit (one result)
+        (True, 10, 5, 1),
+        # don't hit result limit (no results)
+        (True, 2, 1, 0),
+    ],
+)
+def test_session_12(mocker, harness, iters, result_limit, results):
+    """test Session - limit results"""
+    adapter = SimpleAdapter(harness)
+    reporter = mocker.Mock(spec_set=Reporter)
+    report = mocker.Mock(spec_set=Report, major="abc", minor="def", crash_hash="123")
+    report.crash_info.createShortSignature.return_value = "[@ sig]"
+    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
+    target = mocker.Mock(
+        spec_set=Target,
+        assets=mocker.Mock(spec_set=AssetManager),
+        environ=dict(),
+        launch_timeout=30,
+    )
+    target.monitor.launches = 1
+    # avoid shutdown delay
+    target.monitor.is_healthy.return_value = False
+    type(target).closed = mocker.PropertyMock(return_value=True)
+    target.check_result.side_effect = chain(
+        repeat(Result.FOUND, results), repeat(Result.NONE, iters - results)
+    )
+    target.log_size.return_value = 1
+    target.create_report.return_value = report
+    with Session(adapter, reporter, server, target, relaunch=1) as session:
+        server.serve_path = lambda *a, **kv: (
+            Served.ALL,
+            [session.iomanager.page_name(offset=-1)],
+        )
+        session.run(
+            [], 10, input_path="a.bin", iteration_limit=iters, result_limit=result_limit
+        )
+    if results >= result_limit > iters:
+        # limited by result_limit
+        assert session.status.iteration == result_limit
+    else:
+        assert session.status.iteration == iters
+
+
 def test_log_output_limiter_01(mocker):
     """test LogOutputLimiter.ready() not ready"""
     fake_time = mocker.patch("grizzly.session.time", autospec=True)
