@@ -1,13 +1,12 @@
 # coding=utf-8
-"""
-Loki fuzzing library
-"""
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
+"""
+Loki fuzzing library
+"""
+from .loki import Loki
 
-from .loki import Loki  # noqa
-
-__all__ = ("Loki")
+__all__ = ("Loki",)
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith"]
