@@ -7,15 +7,12 @@ Sapphire HTTP server
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 from .core import Sapphire
-from .job import SERVED_ALL, SERVED_NONE, SERVED_REQUEST, SERVED_TIMEOUT
+from .job import Served
 from .server_map import ServerMap
 
 __all__ = (
     "Sapphire",
-    "SERVED_ALL",
-    "SERVED_NONE",
-    "SERVED_REQUEST",
-    "SERVED_TIMEOUT",
+    "Served",
     "ServerMap",
 )
 __author__ = "Tyson Smith"
