@@ -2,27 +2,28 @@
 """
 Job unit tests
 """
-# pylint: disable=protected-access
 
-import platform
+from pathlib import Path
+from platform import system
 
-import pytest
+from pytest import mark, raises
 
-from .job import SERVED_ALL, SERVED_NONE, SERVED_REQUEST, Job
+from .job import Job, Served
 from .server_map import Resource, ServerMap
 
 
 def test_job_01(tmp_path):
     """test creating an empty Job"""
-    job = Job(str(tmp_path))
+    job = Job(tmp_path)
     assert not job.forever
-    assert job.status == SERVED_ALL
-    assert job.check_request("") is None
-    assert job.check_request("test") is None
-    assert job.check_request("test/test/") is None
-    assert job.check_request("test/../../") is None
-    assert not job.is_forbidden(str(tmp_path))
-    assert not job.is_forbidden(str(tmp_path / "missing_file"))
+    assert job.status == Served.NONE
+    assert job.lookup_resource("") is None
+    assert job.lookup_resource("test") is None
+    assert job.lookup_resource("test/test/") is None
+    assert job.lookup_resource("test/../../") is None
+    assert job.lookup_resource("\x00\x0B\xAD\xF0\x0D") is None
+    assert not job.is_forbidden(tmp_path)
+    assert not job.is_forbidden(tmp_path / "missing_file")
     assert job.pending == 0
     assert not job.is_complete()
     assert job.remove_pending("no_file.test")
@@ -43,32 +44,36 @@ def test_job_02(tmp_path):
     req2_path = tmp_path / "nested" / "req_file_2.txt"
     req2_path.write_bytes(b"d")
     job = Job(
-        str(tmp_path), optional_files=[opt1_path.name, "nested/%s" % (opt2_path.name,)]
+        tmp_path, optional_files=[opt1_path.name, "nested/%s" % (opt2_path.name,)]
     )
-    assert job.status == SERVED_NONE
+    assert job.status == Served.NONE
     assert not job.is_complete()
-    resource = job.check_request("req_file_1.txt")
+    resource = job.lookup_resource("req_file_1.txt")
     assert resource.required
     assert job.pending == 2
-    assert resource.target == str(tmp_path / "req_file_1.txt")
+    assert resource.target == tmp_path / "req_file_1.txt"
     assert resource.type == Resource.URL_FILE
-    assert not job.is_forbidden(str(req1_path))
+    assert not job.is_forbidden(req1_path)
     assert not job.remove_pending("no_file.test")
     assert job.pending == 2
     assert not job.remove_pending(str(req1_path))
-    assert job.status == SERVED_REQUEST
+    job.mark_served(req1_path)
+    assert job.status == Served.REQUEST
     assert job.pending == 1
     assert job.remove_pending(str(req2_path))
-    assert job.status == SERVED_ALL
+    job.mark_served(req2_path)
+    assert job.status == Served.ALL
     assert job.pending == 0
     assert job.remove_pending(str(req1_path))
-    resource = job.check_request("opt_file_1.txt")
+    job.mark_served(req1_path)
+    resource = job.lookup_resource("opt_file_1.txt")
     assert not resource.required
-    assert resource.target == str(opt1_path)
+    assert resource.target == opt1_path
     assert resource.type == Resource.URL_FILE
     assert job.remove_pending(str(opt1_path))
-    resource = job.check_request("nested/opt_file_2.txt")
-    assert resource.target == str(opt2_path)
+    job.mark_served(opt1_path)
+    resource = job.lookup_resource("nested/opt_file_2.txt")
+    assert resource.target == opt2_path
     assert resource.type == Resource.URL_FILE
     assert not resource.required
     job.finish()
@@ -80,11 +85,11 @@ def test_job_03(tmp_path):
     smap = ServerMap()
     smap.set_redirect("one", "somefile.txt", required=False)
     smap.set_redirect("two", "reqfile.txt")
-    job = Job(str(tmp_path), server_map=smap)
-    assert job.status == SERVED_NONE
-    resource = job.check_request("one")
+    job = Job(tmp_path, server_map=smap)
+    assert job.status == Served.NONE
+    resource = job.lookup_resource("one")
     assert resource.type == Resource.URL_REDIRECT
-    resource = job.check_request("two?q=123")
+    resource = job.lookup_resource("two")
     assert resource is not None
     assert resource.type == Resource.URL_REDIRECT
     assert job.pending == 1
@@ -116,37 +121,45 @@ def test_job_04(mocker, tmp_path):
     smap = ServerMap()
     smap.set_include("testinc", str(srv_include))
     # add manually to avoid sanity checks in ServerMap.set_include()
-    smap.include["testinc/fakedir"] = Resource(Resource.URL_INCLUDE, str(srv_include))
-    smap.include["testinc/1/2/3"] = Resource(Resource.URL_INCLUDE, str(srv_include))
-    smap.include[""] = Resource(Resource.URL_INCLUDE, str(srv_include))
+    smap.include["testinc/fakedir"] = Resource(Resource.URL_INCLUDE, srv_include)
+    smap.include["testinc/1/2/3"] = Resource(Resource.URL_INCLUDE, srv_include)
+    smap.include[""] = Resource(Resource.URL_INCLUDE, srv_include)
     smap.set_include("testinc/inc2", str(srv_include_2))
-    job = Job(str(srv_root), server_map=smap)
-    assert job.status == SERVED_NONE
+    job = Job(srv_root, server_map=smap)
+    assert job.status == Served.NONE
+    # test include path pointing to a missing file
+    assert job.lookup_resource("testinc/missing") is None
+    # test include path pointing to an invalid file name
+    assert job.lookup_resource("testinc/\x00\x0B\xAD\xF0\x0D") is None
     # test includes that map to 'srv_include'
     for incl, inc_path in smap.include.items():
         if inc_path != str(srv_include):  # only check 'srv_include' mappings
             continue
-        resource = job.check_request("/".join([incl, "test_file.txt"]))
+        resource = job.lookup_resource("/".join([incl, "test_file.txt"]))
         assert resource.type == Resource.URL_INCLUDE
-        assert resource.target == str(inc_1)
+        assert resource.target == inc_1
     # test nested include path pointing to a different include
-    resource = job.check_request("testinc/inc2/test_file_2.txt?q=123")
+    resource = job.lookup_resource("testinc/inc2/test_file_2.txt")
     assert resource.type == Resource.URL_INCLUDE
-    assert resource.target == str(inc_2)
+    assert resource.target == inc_2
     # test redirect root without leading '/'
-    resource = job.check_request("test_file.txt")
+    resource = job.lookup_resource("test_file.txt")
     assert resource.type == Resource.URL_INCLUDE
-    assert resource.target == str(srv_include / "test_file.txt")
+    assert resource.target == srv_include / "test_file.txt"
     # test redirect with file in a nested directory
-    resource = job.check_request("/".join(["testinc", "nested", "nested_file.txt"]))
+    resource = job.lookup_resource("/".join(["testinc", "nested", "nested_file.txt"]))
     assert resource.type == Resource.URL_INCLUDE
-    assert resource.target == str(nst_1)
-    assert not job.is_forbidden(str(srv_root / ".." / "test" / "test_file.txt"))
-    assert not job.is_forbidden(str(srv_include / ".." / "root" / "req_file.txt"))
+    assert resource.target == nst_1
+    assert not job.is_forbidden(
+        (srv_root / ".." / "test" / "test_file.txt").resolve(), is_include=True
+    )
+    assert not job.is_forbidden(
+        (srv_include / ".." / "root" / "req_file.txt").resolve(), is_include=False
+    )
 
 
 def test_job_05(tmp_path):
-    """test Job.check_request() with tricky includes"""
+    """test Job.lookup_resource() with tricky includes"""
     srv_root = tmp_path / "root"
     srv_root.mkdir()
     req = srv_root / "req_file.txt"
@@ -161,12 +174,12 @@ def test_job_05(tmp_path):
     # test url matching part of the file name
     smap = ServerMap()
     smap.include["inc"] = Resource(Resource.URL_INCLUDE, str(inc_dir))
-    job = Job(str(srv_root), server_map=smap)
-    resource = job.check_request("inc/sub/include.js")
+    job = Job(srv_root, server_map=smap)
+    resource = job.lookup_resource("inc/sub/include.js")
     assert resource.type == Resource.URL_INCLUDE
-    assert resource.target == str(inc_file1)
+    assert resource.target == inc_file1
     # test checking only the include url
-    assert job.check_request("inc") is None
+    assert job.lookup_resource("inc") is None
     # file and include file collision (files should always win)
     smap.include.clear()
     inc_a = inc_dir / "a.bin"
@@ -174,37 +187,28 @@ def test_job_05(tmp_path):
     file_a = srv_root / "a.bin"
     file_a.write_bytes(b"a")
     smap.include["/"] = Resource(Resource.URL_INCLUDE, str(inc_dir))
-    resource = job.check_request("a.bin")
+    resource = job.lookup_resource("a.bin")
     assert resource.type == Resource.URL_FILE
-    assert resource.target == str(file_a)
-    # inc and inc subdir collision
-    # TODO: This can fail. How do we detect or support it?
-    # smap.include.clear()
-    # (inc_dir / "c").mkdir()
-    # inc_c_d = (inc_dir / "c" / "d.bin")
-    # inc_c_d.write_bytes(b"a")
-    # inc_d = (inc_dir / "d.bin")
-    # inc_d.write_bytes(b"a")
-    # smap.include["c"] = Resource(Resource.URL_INCLUDE, str(inc_dir))
-    # smap.include[""] = Resource(Resource.URL_INCLUDE, str(inc_dir / "c"))
-    # resource = job.check_request("c/d.bin")
-    # assert resource.type == Resource.URL_INCLUDE
-    # assert resource.target == str(inc_c_d)
+    assert resource.target == file_a
+    # TODO: inc and inc subdir collision can fail.
+    # /inc/a file
+    # /inc   a/file
+    # Should we detect and support it? Does it matter?
 
 
 def test_job_06(tmp_path):
     """test Job dynamic"""
     smap = ServerMap()
-    smap.set_dynamic_response("cb1", lambda: 0, mime_type="mime_type")
-    smap.set_dynamic_response("cb2", lambda: 1)
-    job = Job(str(tmp_path), server_map=smap)
-    assert job.status == SERVED_ALL
+    smap.set_dynamic_response("cb1", lambda _: 0, mime_type="mime_type")
+    smap.set_dynamic_response("cb2", lambda _: 1)
+    job = Job(tmp_path, server_map=smap)
+    assert job.status == Served.NONE
     assert job.pending == 0
-    resource = job.check_request("cb1")
+    resource = job.lookup_resource("cb1")
     assert resource.type == Resource.URL_DYNAMIC
     assert callable(resource.target)
     assert isinstance(resource.mime, str)
-    resource = job.check_request("cb2?q=123")
+    resource = job.lookup_resource("cb2")
     assert resource is not None
     assert resource.type == Resource.URL_DYNAMIC
     assert callable(resource.target)
@@ -219,41 +223,43 @@ def test_job_07(tmp_path):
     test_1.write_bytes(b"a")
     no_access = tmp_path / "no_access.txt"
     no_access.write_bytes(b"a")
-    job = Job(str(srv_root))
-    assert job.status == SERVED_NONE
+    job = Job(srv_root)
+    assert job.status == Served.NONE
     assert job.pending == 1
-    resource = job.check_request("../no_access.txt")
-    assert resource.target == str(no_access)
+    resource = job.lookup_resource("../no_access.txt")
+    assert resource.target == no_access
     assert resource.type == Resource.URL_FILE
-    assert not job.is_forbidden(str(test_1))
-    assert job.is_forbidden(str(srv_root / "../no_access.txt"))
+    assert not job.is_forbidden(test_1)
+    assert job.is_forbidden((srv_root / ".." / "no_access.txt").resolve())
 
 
-@pytest.mark.skipif(platform.system() == "Windows", reason="Unsupported on Windows")
+@mark.skipif(system() == "Windows", reason="Unsupported on Windows")
 def test_job_08(tmp_path):
     """test Job with file names containing invalid characters"""
     test_file = tmp_path / "test.txt"
     test_file.write_bytes(b"a")
     (tmp_path / "?_2.txt").write_bytes(b"a")
-    job = Job(str(tmp_path))
-    assert job.status == SERVED_NONE
+    job = Job(tmp_path)
+    assert job.status == Served.NONE
     assert job.pending == 1
-    assert job.check_request("test.txt").target == str(test_file)
+    assert job.lookup_resource("test.txt").target == test_file
 
 
-def test_job_09():
+def test_job_09(tmp_path):
     """test Job with missing directory"""
-    with pytest.raises(OSError):
-        Job("missing")
+    with raises(OSError):
+        Job(tmp_path / "missing")
 
 
 def test_job_10(tmp_path):
-    """test Job.increment_served() and Job.served"""
-    job = Job(str(tmp_path))
+    """test Job.mark_served() and Job.served"""
+    job = Job(tmp_path)
     assert not any(job.served)
-    job.increment_served(str(tmp_path / "file.bin"))
-    assert "file.bin" in job.served
-    job.increment_served("/some/include/path/inc.bin")
+    job.mark_served(tmp_path / "a.bin")
+    assert "a.bin" in job.served
+    job.mark_served(tmp_path / "nested" / "b.bin")
+    assert "nested/b.bin" in job.served
+    job.mark_served(Path("/some/include/path/inc.bin"))
     assert "/some/include/path/inc.bin" in job.served
 
 
