@@ -9,33 +9,40 @@ import os
 import random
 import socket
 import threading
+from platform import system
+from urllib.parse import quote, urlparse
 
-import pytest
+from pytest import mark, raises
 
 from .core import Sapphire
-from .job import SERVED_ALL, SERVED_NONE, SERVED_REQUEST, SERVED_TIMEOUT
+from .job import Served
 from .server_map import ServerMap
 from .worker import Worker
 
 
 class _TestFile:
-    def __init__(self, url):
+    def __init__(self, url, url_prefix=None):
         self.code = None
         self.content_type = None
         self.custom_request = None
+        if url_prefix:
+            self.file = "".join((url_prefix, url))
+        else:
+            self.file = url
         self.len_org = 0  # original file length
         self.len_srv = 0  # served file length
         self.lock = threading.Lock()
         self.md5_org = None
         self.md5_srv = None
         self.requested = 0  # number of time file was requested
-        self.url = url
+        url = urlparse(self.file.replace("\\", "/"))
+        self.url = (
+            "?".join((quote(url.path), url.query)) if url.query else quote(url.path)
+        )
 
 
 def _create_test(fname, path, data=b"Test!", calc_hash=False, url_prefix=None):
-    test = _TestFile(fname)
-    if url_prefix is not None:
-        test.url = "".join([url_prefix, fname])
+    test = _TestFile(fname, url_prefix=url_prefix)
     with (path / fname).open("w+b") as out_fp:
         out_fp.write(data)
         test.len_org = out_fp.tell()
@@ -51,7 +58,7 @@ def test_sapphire_00(client, tmp_path):
         assert serv.timeout == 10
         test = _create_test("test_case.html", tmp_path)
         client.launch("127.0.0.1", serv.port, [test])
-        assert serv.serve_path(str(tmp_path))[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path)[0] == Served.ALL
     assert client.wait(timeout=10)
     assert test.code == 200
     assert test.len_srv == test.len_org
@@ -68,8 +75,8 @@ def test_sapphire_01(client, tmp_path):
         )
     with Sapphire(timeout=30) as serv:
         client.launch("127.0.0.1", serv.port, to_serve)
-        status, files_served = serv.serve_path(str(tmp_path))
-    assert status == SERVED_ALL
+        status, files_served = serv.serve_path(tmp_path)
+    assert status == Served.ALL
     assert len(to_serve) == len(files_served)
     assert client.wait(timeout=10)
     for t_file in to_serve:
@@ -83,11 +90,11 @@ def test_sapphire_02(client, tmp_path):
     files_to_serve = list()
     for i in range(3):
         files_to_serve.append(_create_test("test_case_%d.html" % i, tmp_path))
-    optional = [files_to_serve[0].url]
+    optional = [files_to_serve[0].file]
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, files_to_serve, in_order=True)
-        status, served_list = serv.serve_path(str(tmp_path), optional_files=optional)
-    assert status == SERVED_ALL
+        status, served_list = serv.serve_path(tmp_path, optional_files=optional)
+    assert status == Served.ALL
     assert len(files_to_serve) == len(served_list)
     assert client.wait(timeout=10)
     for t_file in files_to_serve:
@@ -100,11 +107,11 @@ def test_sapphire_03(client, tmp_path):
     files_to_serve = list()
     for i in range(3):
         files_to_serve.append(_create_test("test_case_%d.html" % i, tmp_path))
-    optional = [files_to_serve[0].url]
+    optional = [files_to_serve[0].file]
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, files_to_serve[1:])
-        status, served_list = serv.serve_path(str(tmp_path), optional_files=optional)
-    assert status == SERVED_ALL
+        status, served_list = serv.serve_path(tmp_path, optional_files=optional)
+    assert status == Served.ALL
     assert len(served_list) == len(files_to_serve) - 1
     assert client.wait(timeout=10)
     assert files_to_serve[0].code is None
@@ -121,9 +128,9 @@ def test_sapphire_04(client, tmp_path):
     files_to_serve.append(_create_test("test_case.html", tmp_path))
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, files_to_serve, in_order=True)
-        assert serv.serve_path(str(tmp_path))[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path)[0] == Served.ALL
     assert client.wait(timeout=10)
-    assert "does_not_exist.html" in files_to_serve[0].url
+    assert "does_not_exist.html" in files_to_serve[0].file
     assert files_to_serve[0].code == 404
     assert files_to_serve[1].code == 200
 
@@ -144,13 +151,13 @@ def test_sapphire_05(client, tmp_path):
     files_to_serve.append(_create_test("test_case.html", root_dir))
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, files_to_serve, in_order=True)
-        status, files_served = serv.serve_path(str(root_dir))
-    assert status == SERVED_ALL
+        status, files_served = serv.serve_path(root_dir)
+    assert status == Served.ALL
     assert len(files_served) == 1
     assert client.wait(timeout=10)
-    assert os.path.basename(__file__) in files_to_serve[0].url
+    assert os.path.basename(__file__) in files_to_serve[0].file
     assert files_to_serve[0].code == 403
-    assert "no_access.html" in files_to_serve[1].url
+    assert "no_access.html" in files_to_serve[1].file
     assert files_to_serve[1].code == 403
     assert files_to_serve[2].code == 200
 
@@ -159,8 +166,8 @@ def test_sapphire_06(client, tmp_path):
     """test serving no files... this should never happen but..."""
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, [])
-        status, files_served = serv.serve_path(str(tmp_path))
-    assert status == SERVED_NONE
+        status, files_served = serv.serve_path(tmp_path)
+    assert status == Served.NONE
     assert not files_served
 
 
@@ -176,13 +183,13 @@ def test_sapphire_07(tmp_path):
         assert serv.timeout == 1
         _create_test("test_case.html", tmp_path)
         serv._timeout = 0.01  # force shorter timeout for faster tests
-        status, files_served = serv.serve_path(str(tmp_path))
-    assert status == SERVED_TIMEOUT
+        status, files_served = serv.serve_path(tmp_path)
+    assert status == Served.TIMEOUT
     assert not files_served
 
 
 def test_sapphire_08(client, tmp_path):
-    """test only serving some files (SERVED_REQUEST)"""
+    """test only serving some files (Served.REQUEST)"""
     cb_status = {"count": 0}
 
     def is_running():
@@ -194,8 +201,8 @@ def test_sapphire_08(client, tmp_path):
         files_to_serve.append(_create_test("test_case_%d.html" % i, tmp_path))
     with Sapphire() as serv:
         client.launch("127.0.0.1", serv.port, files_to_serve[1:])
-        status, files_served = serv.serve_path(str(tmp_path), continue_cb=is_running)
-    assert status == SERVED_REQUEST
+        status, files_served = serv.serve_path(tmp_path, continue_cb=is_running)
+    assert status == Served.REQUEST
     assert len(files_served) < len(files_to_serve)
 
 
@@ -214,12 +221,12 @@ def test_sapphire_09(client, tmp_path):
         t_data = "".join(random.choice("ABCD1234") for _ in range(test["size"])).encode(
             "ascii"
         )
-        (tmp_path / test["file"].url).write_bytes(t_data)
+        (tmp_path / test["file"].file).write_bytes(t_data)
         test["file"].md5_org = hashlib.md5(t_data).hexdigest()
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, [test["file"] for test in tests])
-        status, served_list = serv.serve_path(str(tmp_path))
-    assert status == SERVED_ALL
+        status, served_list = serv.serve_path(tmp_path)
+    assert status == Served.ALL
     assert len(served_list) == len(tests)
     assert client.wait(timeout=10)
     for test in tests:
@@ -232,7 +239,7 @@ def test_sapphire_10(client, tmp_path):
     """test serving a large (100MB) file"""
     t_file = _TestFile("test_case.html")
     data_hash = hashlib.md5()
-    with (tmp_path / t_file.url).open("wb") as test_fp:
+    with (tmp_path / t_file.file).open("wb") as test_fp:
         # write 100MB of 'A'
         data = b"A" * (100 * 1024)  # 100KB of 'A'
         for _ in range(1024):
@@ -241,7 +248,7 @@ def test_sapphire_10(client, tmp_path):
     t_file.md5_org = data_hash.hexdigest()
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, [t_file])
-        assert serv.serve_path(str(tmp_path))[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path)[0] == Served.ALL
     assert client.wait(timeout=10)
     assert t_file.code == 200
     assert t_file.len_srv == (100 * 1024 * 1024)
@@ -255,7 +262,7 @@ def test_sapphire_11(client, tmp_path):
     )
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, [t_file])
-        assert serv.serve_path(str(tmp_path))[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path)[0] == Served.ALL
     assert client.wait(timeout=10)
     assert t_file.code == 200
     assert t_file.len_srv == t_file.len_org
@@ -277,13 +284,13 @@ def test_sapphire_13(client, tmp_path):
     files_to_serve.append(_create_test("test_case", tmp_path, data=os.urandom(5)))
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, files_to_serve)
-        assert serv.serve_path(str(tmp_path))[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path)[0] == Served.ALL
     assert client.wait(timeout=10)
     content_types = set()
     for test in files_to_serve:
         assert test.code == 200
         assert test.len_srv == test.len_org
-        file_ext = os.path.splitext(test.url)[-1]
+        file_ext = os.path.splitext(test.file)[-1]
         content_type = {".html": "text/html"}.get(file_ext, "application/octet-stream")
         content_types.add(content_type)
         assert test.content_type == content_type
@@ -301,9 +308,7 @@ def test_sapphire_14(tmp_path):
 
     with Sapphire(timeout=10) as serv:
         _create_test("test_case.html", tmp_path)
-        assert (
-            serv.serve_path(str(tmp_path), continue_cb=_test_callback)[0] == SERVED_NONE
-        )
+        assert serv.serve_path(tmp_path, continue_cb=_test_callback)[0] == Served.NONE
     assert cb_status["count"] == 2
 
 
@@ -313,12 +318,12 @@ def test_sapphire_15(client, tmp_path):
         for i in range(3):
             test = _create_test("test_case_%d.html" % i, tmp_path)
             client.launch("127.0.0.1", serv.port, [test])
-            assert serv.serve_path(str(tmp_path))[0] == SERVED_ALL
+            assert serv.serve_path(tmp_path)[0] == Served.ALL
             assert client.wait(timeout=10)
             client.close()
             assert test.code == 200
             assert test.len_srv == test.len_org
-            (tmp_path / test.url).unlink()
+            (tmp_path / test.file).unlink()
 
 
 def test_sapphire_16(client, tmp_path):
@@ -328,35 +333,42 @@ def test_sapphire_16(client, tmp_path):
     test = _create_test("test_case.html", tmp_path)
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, [test])
-        assert serv.serve_path(str(tmp_path), server_map=smap)[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path, server_map=smap)[0] == Served.ALL
     assert client.wait(timeout=10)
     assert test.code == 200
     assert test.len_srv == test.len_org
 
 
-def test_sapphire_17(client, tmp_path):
+@mark.parametrize(
+    "path, query",
+    [
+        # simple path
+        ("test.html", None),
+        # simple path with query
+        ("test.html", "foo=1&bar=2"),
+        # non-alphanumeric chars (valid characters to use on filesystem)
+        ("!@$%^&(_+-=[]),'~`{}", None),
+        # extended ascii chars
+        ("€d’é-ñÿ", None),
+    ],
+)
+def test_sapphire_17(client, tmp_path, path, query):
     """test required mapped redirects"""
     smap = ServerMap()
     with Sapphire(timeout=10) as serv:
-        files_to_serve = list()
-        # redir_target will be requested indirectly via the redirect
-        redir_target = _create_test(
-            "redir_test_case.html", tmp_path, data=b"Redirect DATA!"
-        )
-        redir_test = _TestFile("redirect_test")
-        smap.set_redirect(redir_test.url, redir_target.url, required=True)
-        files_to_serve.append(redir_test)
-        test = _create_test("test_case.html", tmp_path)
-        files_to_serve.append(test)
-        client.launch("127.0.0.1", serv.port, files_to_serve)
-        status, served_list = serv.serve_path(str(tmp_path), server_map=smap)
-    assert status == SERVED_ALL
-    assert len(served_list) == len(files_to_serve)
+        # target will be requested indirectly via the redirect
+        target = _create_test(path, tmp_path, data=b"Redirect DATA!")
+        request_path = "redirect" if query is None else "?".join(("redirect", query))
+        redirect = _TestFile(request_path)
+        # point "redirect" at target
+        smap.set_redirect("redirect", target.file, required=True)
+        client.launch("127.0.0.1", serv.port, [redirect])
+        status, served = serv.serve_path(tmp_path, server_map=smap)
+    assert status == Served.ALL
+    assert len(served) == 1
     assert client.wait(timeout=10)
-    assert test.code == 200
-    assert test.len_srv == test.len_org
-    assert redir_test.code == 200
-    assert redir_test.len_srv == redir_target.len_org
+    assert redirect.code == 200
+    assert redirect.len_srv == target.len_org
 
 
 def test_sapphire_18(client, tmp_path):
@@ -407,12 +419,12 @@ def test_sapphire_18(client, tmp_path):
         smap.set_include("/", str(inc1_path))  # mount at '/'
         smap.set_include("inc_test", str(inc2_path))  # mount at '/inc_test'
         client.launch("127.0.0.1", serv.port, files_to_serve, in_order=True)
-        status, files_served = serv.serve_path(str(root_path), server_map=smap)
-    assert status == SERVED_ALL
+        status, files_served = serv.serve_path(root_path, server_map=smap)
+    assert status == Served.ALL
     assert "test_case.html" in files_served
-    assert str(inc1_path / "included_file1.html") in files_served
-    assert str(inc2_path / "included_file2.html") in files_served
-    assert str(nest_path / "nested_file.html") in files_served
+    assert (inc1_path / "included_file1.html").as_posix() in files_served
+    assert (inc2_path / "included_file2.html").as_posix() in files_served
+    assert (nest_path / "nested_file.html").as_posix() in files_served
     assert client.wait(timeout=10)
     assert inc1.code == 200
     assert inc2.code == 200
@@ -423,51 +435,74 @@ def test_sapphire_18(client, tmp_path):
     assert inc403.code == 403
 
 
-def test_sapphire_19(client, tmp_path):
-    """test dynamic response - not required"""
+@mark.parametrize(
+    "query, required",
+    [
+        # dynamic response not required
+        (None, False),
+        # dynamic response is required
+        (None, True),
+        # dynamic response with query string
+        ("test123", True),
+        # dynamic response with empty query string
+        ("", True),
+    ],
+)
+def test_sapphire_19(client, tmp_path, query, required):
+    """test dynamic response"""
     _data = b"dynamic response -- TEST DATA!"
-    test_dr = _TestFile("dyn_test")
-    test_dr.len_org = len(_data)
-    test_dr.md5_org = hashlib.md5(_data).hexdigest()
-    smap = ServerMap()
-    smap.set_dynamic_response("dyn_test", lambda: _data, mime_type="text/plain")
-    test = _create_test("test_case.html", tmp_path)
-    with Sapphire(timeout=10) as serv:
-        client.launch("127.0.0.1", serv.port, [test_dr, test], in_order=True)
-        assert serv.serve_path(str(tmp_path), server_map=smap)[0] == SERVED_ALL
-    assert client.wait(timeout=10)
-    assert test.code == 200
-    assert test.len_srv == test.len_org
-    assert test_dr.code == 200
-    assert test_dr.len_srv == test_dr.len_org
-    assert test_dr.md5_srv == test_dr.md5_org
+    # build request
+    path = "dyn_test"
+    if query is not None:
+        request = "?".join([path, query])
+    else:
+        request = path
 
+    # setup custom callback
+    def dr_callback(data):
+        if query:
+            assert data == query
+        else:
+            assert data == ""
+        return _data
 
-def test_sapphire_20(client, tmp_path):
-    """test dynamic response - required"""
-    _data = b"dynamic response -- TEST DATA!"
-    test_dr = _TestFile("dyn_test")
-    test_dr.len_org = len(_data)
-    test_dr.md5_org = hashlib.md5(_data).hexdigest()
     smap = ServerMap()
     smap.set_dynamic_response(
-        "dyn_test", lambda: _data, mime_type="text/plain", required=True
+        path, dr_callback, mime_type="text/plain", required=required
     )
+    # create files
+    test_dr = _TestFile(request)
+    test_dr.len_org = len(_data)
+    test_dr.md5_org = hashlib.md5(_data).hexdigest()
+    test = _create_test("test_case.html", tmp_path)
+    if required:
+        optional = [test.file]
+        files = [test, test_dr]
+    else:
+        optional = [test_dr.file]
+        files = [test_dr, test]
+    # test request
     with Sapphire(timeout=10) as serv:
-        client.launch("127.0.0.1", serv.port, [test_dr], in_order=True)
-        assert serv.serve_path(str(tmp_path), server_map=smap)[0] == SERVED_ALL
+        client.launch("127.0.0.1", serv.port, files, in_order=True)
+        assert (
+            serv.serve_path(tmp_path, optional_files=optional, server_map=smap)[0]
+            == Served.ALL
+        )
     assert client.wait(timeout=10)
+    if not required:
+        assert test.code == 200
+        assert test.len_srv == test.len_org
     assert test_dr.code == 200
     assert test_dr.len_srv == test_dr.len_org
     assert test_dr.md5_srv == test_dr.md5_org
 
 
-def test_sapphire_21(client_factory, tmp_path):
+def test_sapphire_20(client_factory, tmp_path):
     """test pending_files == 0 in worker thread"""
     client_defer = client_factory(rx_size=2)
     # server should shutdown while this file is being served
     test_defer = _create_test("defer_test.html", tmp_path)
-    optional = [test_defer.url]
+    optional = [test_defer.file]
     test = _create_test("test_case.html", tmp_path, data=b"112233")
     with Sapphire(timeout=10) as serv:
         # this test needs to wait just long enough to have the required file served
@@ -477,32 +512,32 @@ def test_sapphire_21(client_factory, tmp_path):
         )
         client = client_factory(rx_size=2)
         client.launch("127.0.0.1", serv.port, [test], throttle=0.1)
-        assert serv.serve_path(str(tmp_path), optional_files=optional)[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path, optional_files=optional)[0] == Served.ALL
     assert client_defer.wait(timeout=10)
     assert client.wait(timeout=10)
     assert test.code == 200
     assert test_defer.code == 0
 
 
-def test_sapphire_22(client, tmp_path):
+def test_sapphire_21(client, tmp_path):
     """test handling an invalid request"""
     bad_test = _TestFile("bad.html")
     bad_test.custom_request = b"a bad request...0+%\xef\xb7\xba\r\n"
-    optional = [bad_test.url]
+    optional = [bad_test.file]
     test = _create_test("test_case.html", tmp_path)
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, [bad_test, test], in_order=True)
-        assert serv.serve_path(str(tmp_path), optional_files=optional)[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path, optional_files=optional)[0] == Served.ALL
     assert client.wait(timeout=10)
     assert test.code == 200
     assert bad_test.code == 400
 
 
-def test_sapphire_23(client, tmp_path):
+def test_sapphire_22(client, tmp_path):
     """test handling an empty request"""
     bad_test = _TestFile("bad.html")
     bad_test.custom_request = b""
-    optional = [bad_test.url]
+    optional = [bad_test.file]
     test = _create_test("test_case.html", tmp_path)
     with Sapphire(timeout=10) as serv:
         client.launch(
@@ -512,13 +547,13 @@ def test_sapphire_23(client, tmp_path):
             indicate_failure=True,
             in_order=True,
         )
-        assert serv.serve_path(str(tmp_path), optional_files=optional)[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path, optional_files=optional)[0] == Served.ALL
     assert client.wait(timeout=10)
     assert test.code == 200
     assert bad_test.code == 0
 
 
-def test_sapphire_24(client_factory, tmp_path):
+def test_sapphire_23(client_factory, tmp_path):
     """test requesting multiple files via multiple connections"""
     to_serve = list()
     for i in range(2):
@@ -533,24 +568,24 @@ def test_sapphire_24(client_factory, tmp_path):
                 client.launch(
                     "127.0.0.1", serv.port, to_serve, in_order=True, throttle=0.05
                 )
-            status, files_served = serv.serve_path(str(tmp_path))
+            status, files_served = serv.serve_path(tmp_path)
             # call serv.close() instead of waiting for the clients to timeout
             serv.close()
         finally:
             for client in clients:
                 assert client.wait(timeout=10)
                 client.close()
-    assert status == SERVED_ALL
+    assert status == Served.ALL
     assert len(to_serve) == len(files_served)
     for t_file in to_serve:
         assert t_file.code == 200
         assert t_file.len_srv == t_file.len_org
 
 
-def test_sapphire_25(client_factory, tmp_path):
+def test_sapphire_24(client_factory, tmp_path):
     """test all request types via multiple connections"""
 
-    def _dyn_test_cb():
+    def _dyn_test_cb(_):
         return b"A" if random.getrandbits(1) else b"AA"
 
     smap = ServerMap()
@@ -571,34 +606,34 @@ def test_sapphire_25(client_factory, tmp_path):
             redir_target = _create_test("redir_%03d.html" % i, tmp_path, data=b"AA")
             to_serve.append(_TestFile("redir_%03d" % i))
             smap.set_redirect(
-                to_serve[-1].url, redir_target.url, required=random.getrandbits(1) > 0
+                to_serve[-1].file, redir_target.file, required=random.getrandbits(1) > 0
             )
             # add dynamic responses
             to_serve.append(_TestFile("dynm_%03d" % i))
             smap.set_dynamic_response(
-                to_serve[-1].url, _dyn_test_cb, mime_type="text/plain"
+                to_serve[-1].file, _dyn_test_cb, mime_type="text/plain"
             )
         clients = list()
         for _ in range(100):  # number of clients to spawn
             clients.append(client_factory(rx_size=1))
             throttle = 0.05 if random.getrandbits(1) else 0
             clients[-1].launch("127.0.0.1", serv.port, to_serve, throttle=throttle)
-        assert serv.serve_path(str(tmp_path), server_map=smap)[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path, server_map=smap)[0] == Served.ALL
 
 
-def test_sapphire_26(client, tmp_path):
+def test_sapphire_25(client, tmp_path):
     """test dynamic response with bad callbacks"""
     test_dr = _TestFile("dynm_test")
     smap = ServerMap()
-    smap.set_dynamic_response("dynm_test", lambda: None, mime_type="text/plain")
+    smap.set_dynamic_response("dynm_test", lambda _: None, mime_type="text/plain")
     test = _create_test("test_case.html", tmp_path)
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, [test_dr, test], in_order=True)
-        with pytest.raises(TypeError):
-            serv.serve_path(str(tmp_path), server_map=smap)
+        with raises(TypeError):
+            serv.serve_path(tmp_path, server_map=smap)
 
 
-def test_sapphire_27(client, tmp_path):
+def test_sapphire_26(client, tmp_path):
     """test serving to a slow client"""
     t_data = "".join(random.choice("ABCD1234") for _ in range(0x19000))  # 100KB
     t_file = _create_test(
@@ -610,14 +645,14 @@ def test_sapphire_27(client, tmp_path):
     client.rx_size = 0x2800
     with Sapphire(timeout=60) as serv:
         client.launch("127.0.0.1", serv.port, [t_file], throttle=0.25)
-        assert serv.serve_path(str(tmp_path))[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path)[0] == Served.ALL
     assert client.wait(timeout=10)
     assert t_file.code == 200
     assert t_file.len_srv == t_file.len_org
     assert t_file.md5_srv == t_file.md5_org
 
 
-def test_sapphire_28(client, tmp_path):
+def test_sapphire_27(client, tmp_path):
     """test timeout while requesting multiple files"""
     files_to_serve = list()
     t_data = "".join(random.choice("ABCD1234") for _ in range(1024)).encode("ascii")
@@ -630,12 +665,12 @@ def test_sapphire_28(client, tmp_path):
         client.launch(
             "127.0.0.1", serv.port, files_to_serve, indicate_failure=True, throttle=0.1
         )
-        status, files_served = serv.serve_path(str(tmp_path))
-    assert status == SERVED_TIMEOUT
+        status, files_served = serv.serve_path(tmp_path)
+    assert status == Served.TIMEOUT
     assert len(files_served) < len(files_to_serve)
 
 
-def test_sapphire_29(client_factory, tmp_path):
+def test_sapphire_28(client_factory, tmp_path):
     """test Sapphire.serve_path() with forever=True"""
     clients = list()
     with Sapphire(timeout=10) as serv:
@@ -651,8 +686,8 @@ def test_sapphire_29(client_factory, tmp_path):
                 return test.requested < 3
 
         assert (
-            serv.serve_path(str(tmp_path), continue_cb=_test_callback, forever=True)[0]
-            == SERVED_ALL
+            serv.serve_path(tmp_path, continue_cb=_test_callback, forever=True)[0]
+            == Served.ALL
         )
     for client in clients:
         assert client.wait(timeout=10)
@@ -662,40 +697,44 @@ def test_sapphire_29(client_factory, tmp_path):
     assert test.len_srv == test.len_org
 
 
-def test_sapphire_30(client, tmp_path):
-    """test interesting file names"""
-    to_serve = [
+@mark.parametrize(
+    "file_name",
+    [
         # space in file name
-        _create_test("test case.html", tmp_path),
+        "test case.html",
         # non-alphanumeric chars (valid characters to use on filesystem)
-        _create_test("!@#$%^&(_+-=[]),;'~`{}", tmp_path),
-    ]
+        "!@$%^&(_+-=[]),'~`{}",
+        # extended ascii chars
+        "€d’é-ñÿ",
+    ],
+)
+def test_sapphire_29(client, tmp_path, file_name):
+    """test interesting file names"""
+    to_serve = [_create_test(file_name, tmp_path)]
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, to_serve)
-        assert serv.serve_path(str(tmp_path))[0] == SERVED_ALL
+        assert serv.serve_path(tmp_path)[0] == Served.ALL
     assert client.wait(timeout=10)
     assert all(t_file.code == 200 for t_file in to_serve)
 
 
-def test_sapphire_31(client, tmp_path):
+def test_sapphire_30(client, tmp_path):
     """test interesting path string"""
-    all_bytes = "".join(chr(i) for i in range(256))
+    path = "".join(chr(i) for i in range(256))
     to_serve = [
         # should not trigger crash
-        _TestFile(all_bytes),
+        _TestFile(path),
         # used to keep server running
         _create_test("a.html", tmp_path),
     ]
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, to_serve, in_order=True)
-        assert (
-            serv.serve_path(str(tmp_path), optional_files=[all_bytes])[0] == SERVED_ALL
-        )
+        assert serv.serve_path(tmp_path, optional_files=[path])[0] == Served.ALL
     assert client.wait(timeout=10)
     assert all(t_file.code is not None for t_file in to_serve)
 
 
-def test_sapphire_32(mocker):
+def test_sapphire_31(mocker):
     """test Sapphire._create_listening_socket()"""
     fake_sleep = mocker.patch("sapphire.core.sleep", autospec=True)
     fake_sock = mocker.patch("sapphire.core.socket", autospec=True)
@@ -709,7 +748,7 @@ def test_sapphire_32(mocker):
     fake_sock.reset_mock()
     # failure to bind
     fake_sock.return_value.bind.side_effect = OSError
-    with pytest.raises(OSError):
+    with raises(OSError):
         Sapphire._create_listening_socket(False, None)
     assert fake_sock.return_value.close.call_count == 1
     assert fake_sleep.call_count == 0
@@ -724,13 +763,13 @@ def test_sapphire_32(mocker):
     assert fake_sleep.call_count == 1
 
 
-def test_sapphire_33(mocker):
+def test_sapphire_32(mocker):
     """test Sapphire.clear_backlog()"""
     mocker.patch("sapphire.core.socket", autospec=True)
     mocker.patch("sapphire.core.time", autospec=True, return_value=1)
-    pending = mocker.Mock(spec=socket.socket)
+    pending = mocker.Mock(spec_set=socket.socket)
     with Sapphire(timeout=10) as serv:
-        serv._socket = mocker.Mock(spec=socket.socket)
+        serv._socket = mocker.Mock(spec_set=socket.socket)
         serv._socket.accept.side_effect = ((pending, None), OSError, BlockingIOError)
         serv.clear_backlog()
         assert serv._socket.accept.call_count == 3
@@ -738,13 +777,28 @@ def test_sapphire_33(mocker):
     assert pending.close.call_count == 1
 
 
+@mark.skipif(system() != "Windows", reason="Only supported on Windows")
+def test_sapphire_33(client, tmp_path):
+    """test serving from path using Windows short file name"""
+    wwwroot = tmp_path / "long_path_name_that_can_be_truncated_on_windows"
+    wwwroot.mkdir()
+    with Sapphire(timeout=10) as serv:
+        assert serv.timeout == 10
+        test = _create_test("test_case.html", wwwroot)
+        client.launch("127.0.0.1", serv.port, [test])
+        assert serv.serve_path(tmp_path / "LONG_P~1")[0] == Served.ALL
+    assert client.wait(timeout=10)
+    assert test.code == 200
+    assert test.len_srv == test.len_org
+
+
 def test_main_01(mocker, tmp_path):
     """test Sapphire.main()"""
     args = mocker.Mock(path=str(tmp_path), port=4536, remote=False, timeout=None)
     fake_srv = mocker.patch("sapphire.core.Sapphire.serve_path", autospec=True)
-    fake_srv.return_value = (SERVED_ALL, None)
+    fake_srv.return_value = (Served.ALL, None)
     Sapphire.main(args)
-    fake_srv.return_value = (SERVED_NONE, None)
+    fake_srv.return_value = (Served.NONE, None)
     Sapphire.main(args)
     fake_srv.side_effect = KeyboardInterrupt
     Sapphire.main(args)
